/*
Collection of template array functions
Copyright (C) 2002  Daniel Yukio Yokomiso

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

See attached file lesser.txt for a copy of the GNU Lesser General Public License
 */

module deimos.arrays;
import deimos.base;
import std.math;
template TArrays(T) {
    alias TComparable!(int) cmp;
    public T[] head(T[] array, int amount)
    in {
        assert(0 <= amount);
        assert(amount <= array.length);
    } out (result) {
        assert(result.length == amount);
    } body {
        return array[0 .. amount];
    }
    public T[] tail(T[] array, int amount)
    in {
        assert(0 <= amount);
        assert(amount <= array.length);
    } out (result) {
        assert(result.length == amount);
    } body {
        return array [array.length - amount .. array.length];
    }
    public T first(T[] array)
    in {
        assert(array.length > 0);
    } body {
        return array[0];
    }
    public T last(T[] array)
    in {
        assert(array.length > 0);
    } body {
        return array[array.length - 1];
    }
    public T[] rest(T[] array)
    in {
        assert(array.length > 0);
    } body {
        return array[1 .. array.length];
    }
    public T[] asArray(T item) {
        T[] result = new T[1];
        result[0] = item;
        return result;
    }
    public int find(T[] array, T item) {
        return find(array, asArray(item));
    }
    public int find(T[] array, T[] inner) {
        if (inner.length > array.length) {
            return -1;
        } else if (inner.length == 0) {
            return -1;
        }
        for (int i = 0; i < array.length; i++) {
            if (array[i] == first(inner)) {
                if (inner.length > (array.length - i)) {
                    return -1;
                }
                return array[i .. i + inner.length] == inner[] ? i : -1;
            }
        }
        return -1;
    }
    public int rfind(T[] array, T item) {
        return rfind(array, asArray(item));
    }
    public int rfind(T[] array, T[] inner) {
        if (inner.length > array.length) {
            return -1;
        } else if (inner.length == 0) {
            return -1;
        }
        for (int i = array.length - 1; i >= 0; i--) {
            if (array[i] == last(inner)) {
                if (inner.length > (i + 1)) {
                     return -1;
                }
                int start = i - inner.length + 1;
                return array[start .. i + 1] == inner[] ? start : -1;
            }
        }
        return -1;
    }
    public boolean startsWith(T[] array, T item) {
        return startsWith(array, asArray(item));
    }
    public boolean startsWith(T[] array, T[] inner) {
        if (array.length < inner.length) {
            return false;
        } else if (inner.length == 0) {
            return false;
        } else if (array.length > inner.length) {
            return startsWith(head(array, inner.length), inner);
        } else {
            return array[] == inner[];
        }
    }
    public boolean endsWith(T[] array, T item) {
        return endsWith(array, asArray(item));
    }
    public boolean endsWith(T[] array, T[] inner) {
        if (array.length < inner.length) {
            return false;
        } else if (inner.length == 0) {
            return false;
        } else if (array.length > inner.length) {
            return endsWith(tail(array, inner.length), inner);
        } else {
            return array[] == inner[];
        }
    }
    public boolean contains(T[] array, T item) { 
        return contains(array, asArray(item));
    }
    public boolean contains(T[] array, T[] inner) {
        return find(array, inner) > -1;
    }
    public boolean all(T[] array, boolean (*predicate)(T))
    out(result) {
        assert(result == (count(array, predicate) == array.length));
    } body {
        for (int i = 0; i < array.length; i++) {
            if (!predicate(array[i])) {
                return false;
            }
        }
        return true;
    }
    public boolean any(T[] array, boolean (*predicate)(T))
    out(result) {
        assert(result == (count(array, predicate) > 0));
    } body {
        for (int i = 0; i < array.length; i++) {
            if (predicate(array[i])) {
                return true;
            }
        }
        return false;
    }
    public int count(T[] array, boolean (*predicate)(T))
    out(result) {
        assert(result >= 0);
    } body {
        int res = 0;
        for (int i = 0; i < array.length; i++) {
            if (predicate(array[i])) {
                res++;
            }
        }
        return res;
    }
    public T[] filter(T[] array, boolean (*predicate)(T))
    out(result) {
        assert(all(result, predicate));
    } body {
        T[] res;
        for (int i = 0; i < array.length; i++) {
            if (predicate(array[i])) {
                res ~= array[i];
            }
        }
        return res;
    }
    public T max(T[] array, Order (*order)(T, T))
    in {
        assert(array.length > 0);
    } body {
        T result = first(array);
        for (int i = 1; i < array.length; i++) {
            if (order(result, array[i]) == LESS_THAN) {
                result = array[i];
            }
        }
        return result;
    }
    public T min(T[] array, Order (*order)(T, T))
    in {
        assert(array.length > 0);
    } body {
        T result = first(array);
        for (int i = 1; i < array.length; i++) {
            if (order(result, array[i]) == GREATER_THAN) {
                result = array[i];
            }
        }
        return result;
    }
    public boolean isEmpty(T[] array) {
        return array.length == 0;
    }
    public int countOf(T[] array, T item)
    out(result) {
        assert(result >= 0);
    } body {
        return countOf(array, asArray(item));
    }
    public int countOf(T[] array, T[] inner) 
    out(result) {
        assert(result >= 0);
    } body {
        if (inner.length > array.length) {
            return 0;
        } else if (inner.length == 0) {
            return 0;
        }
        int count = 0;
        for (int i = 0; i + inner.length <= array.length;) {
            if (array[i] == first(inner)) {
                if (inner[] == array[i .. i + inner.length]) {
                    count++;
                    i += inner.length;
                } else {
                    i++;
                }
            } else {
                i++;
            }
        }
        return count;
    }
    public T[] repeat(T[] array, int amount)
    in {
        assert(amount >= 0);
    } out(result) {
        assert(result.length == (array.length * amount));
    } body {
        T[] res = new T[array.length * amount];
        for (int i = 0; i < amount; i++) {
            res[i * array.length .. (i + 1) * array.length] = array[];
        }
        return res;
    }
    public T[] zip(T[] left, T[] right)
    out(result) {
        assert(result.length == (left.length + right.length));
    } body {
        if (isEmpty(left)) {
            return right;
        } else if (isEmpty(right)) {
            return left;
        } else {
            T[] res;
            res ~= first(left);
            res ~= first(right);
            res ~= zip(rest(left), rest(right));
            return res;
        }
    }
    T[] join(T[][] arrays, T[] separator) {
        T[] res;
        if (arrays.length > 0) {
            res ~= arrays[0];
            for (int i = 1; i < arrays.length; i++) {
                res ~= separator;
                res ~= arrays[i];
            }
        }
        return res;
    }
    public T at(T[] array, int index) {
        return array [index < 0 ? index + array.length : index];
    }
    public T atWrap(T[] array, int index) {
        return array [((index % array.length) + array.length) % array.length];
    }
    public T atSaturate(T[] array, int index) {
        return array [cmp.clamp(index, 0, array.length - 1)];
    }
    public T atBorder(T[] array, int index, T outside) {
        return index >= 0 && index < array.length ? array [index] : outside;
    }
    public T atReflect(T[] array, int index) {
        int i = index;
        while(true) {
            if (i < 0) {
                i = -i - 1;
            } else if (i >= array.length) {
                i = (array.length - 2) - (i - array.length);
            } else {
                break;
            }
        }
        return array[i];
    }
    public void insert(inout T[] array, int index, T item)
    in {
        assert(index >= 0);
        assert(array.length >= index);
    } body {
        insert(array, index, asArray(item));
    }
    public void insert(inout T[] array, int index, T[] items)
    in {
        assert(index >= 0);
        assert(array.length >= index);
    } body {
        array.length = array.length + items.length;
        for (int i = array.length - 1; i >= (index + items.length); i--) {
            array[i] = array[i - items.length];
        }
        array[index .. index + items.length] = items;
    }
    public T[] inserted(T[] array, int index, T item)
    in {
        assert(index >= 0);
        assert(array.length >= index);
    } out(result) {
        assert(result.length == array.length + 1);
    } body {
        return inserted(array, index, asArray(item));
    }
    public T[] inserted(T[] array, int index, T[] items)
    in {
        assert(index >= 0);
        assert(array.length >= index);
    } out(result) {
        assert(result.length == array.length + items.length);
    } body {
        T[] doppelganger = array.dup;
        insert(doppelganger, index, items);
        return doppelganger;
    }
    public void replace(inout T[] array, T oldItem, T newItem) {
        replace(array, asArray(oldItem), asArray(newItem));
    }
    public void replace(inout T[] array, T[] oldItems, T newItem) {
        replace(array, oldItems, asArray(newItem));
    }
    public void replace(inout T[] array, T oldItem, T[] newItems) {
        replace(array, asArray(oldItem), newItems);
    }
    public void replace(inout T[] array, T[] oldItems, T[] newItems) {
        T[] res;
        if (newItems.length > array.length) {
            return;
        } else if (oldItems.length == 0) {
            return;
        }
        for (int i = 0; i + oldItems.length <= array.length;) {
            if (array[i] == first(oldItems)) {
                if (oldItems[] == array[i .. i + oldItems.length]) {
                    i += oldItems.length;
                    res ~= newItems;
                } else {
                    res ~= array[i];
                    i++;
                }
            } else {
                res ~= array[i];
                i++;
            }
        }
        array = res;
    }
    public T[] replaced(T[] array, T oldItem, T newItem) {
        return replaced(array, asArray(oldItem), asArray(newItem));
    }
    public T[] replaced(T[] array, T[] oldItems, T newItem) {
        return replaced(array, oldItems, asArray(newItem));
    }
    public T[] replaced(T[] array, T oldItem, T[] newItems) {
        return replaced(array, asArray(oldItem), newItems);
    }
    public T[] replaced(T[] array, T[] oldItems, T[] newItems) {
        T[] doppelganger = array.dup;
        replace(doppelganger, oldItems, newItems);
        return doppelganger;
    }
    public void remove(inout T[] array, int index)
    in {
        assert(index >= 0);
        assert(index < array.length);
    } body {
        remove(array, index, index + 1);
    }
    public T[] removed(T[] array, int index)
    in {
        assert(index >= 0);
        assert(index < array.length);
    } out (result) {
        assert(result.length == (array.length - 1));
        assert(head(result, index) == head(array, index));
        assert(tail(result, array.length - index - 1) == tail(array, array.length - index - 1));
    } body {
        T[] doppelganger = array.dup;
        remove(doppelganger, index);
        return doppelganger;
    }
    public void remove(inout T[] array, int start, int afterEnd)
    in {
        assert(start >= 0);
        assert(start < array.length);
        assert(afterEnd >= start);
        assert(afterEnd <= array.length);
    } body {
        int amount = afterEnd - start;
        for (int i = start; i < (array.length - amount); i++) {
            array[i] = array[i + amount];
        }
        array.length = array.length - amount;
    }
    public T[] removed(T[] array, int start, int afterEnd)
    in {
        assert(start >= 0);
        assert(start < array.length);
        assert(afterEnd >= start);
        assert(afterEnd <= array.length);
    } out (result) {
        assert(result.length == (array.length - (afterEnd - start)));
        assert(head(result, start) == head(array, start));
        assert(tail(result, array.length - afterEnd) == tail(array, array.length - afterEnd));
    } body {
        T[] doppelganger = array.dup;
        remove(doppelganger, start, afterEnd);
        return doppelganger;
    }
    public boolean isOrdered(T[] array, Order (*order)(T, T)) {
        if (array.length <= 1) {
            return true;
        }
        for (int i = 0; i < array.length -1; i++) {
            if (order(array[i], array[i + 1]) == GREATER_THAN) {
                return false;
            }
        }
        return true;
    }
    public void sort(inout T[] array, Order (*order)(T, T)) {
        if (array.length > 1) {
            int middle = array.length / 2;
            T[] left = array[0..middle];
            T[] right = array[middle..array.length];
            sort(left, order);
            sort(right, order);
            array = merge(left, right, order);
        }
    }
    public T[] sorted(T[] array, Order (*order)(T, T)) {
        T[] doppelganger = array.dup;
        sort(doppelganger, order);
        return doppelganger;
    }
    public T[] merge(T[] left, T[] right, Order (*order)(T, T))
    in {
        assert(isOrdered(left, order));
        assert(isOrdered(right, order));
    } body {
        if (isEmpty(left)) {
            return right;
        } else if (isEmpty(right)) {
            return left;
        } else {
            T[] res;
            T l = first(left);
            T r = first(right);
            if (order(l, r) < GREATER_THAN) {
                res ~= l;
                res ~= merge(rest(left), right, order);
            } else {
                res ~= r;
                res ~= merge(left, rest(right), order);
            }
            return res;
        }
    }
    public void foreachFn(T[] array, void (*operation) (T)) { /* was foreachFn was foreach */
        if (!isEmpty(array)) {
            operation(first(array));
            foreachFn(rest(array), operation); /* was foreachFn was foreach */
        }
    }
    public void opApply(inout T[] array, T (*operation) (T))
    out {
//        assert(old array.length == array.length);
    } body {
        for (int i = 0; i < array.length; i++) {
            array[i] = operation(array[i]);
        }
    }
}
template TComparableArrays(T) {
    private alias TArrays!(T) arrays;
    private alias TComparable!(T) cmp;
    public T min(T[] array)
    in {
        assert(array.length > 0);
    } body {
        return arrays.min(array, &cmp.ascending);
    }
    public T max(T[] array)
    in {
        assert(array.length > 0);
    } body {
        return arrays.max(array, &cmp.ascending);
    }
    public boolean isOrdered(T[] array) {
        return arrays.isOrdered(array, &cmp.ascending);
    }
    public void sort(inout T[] array) {
        arrays.sort(array, &cmp.ascending);
    }
    public T[] sorted(T[] array) {
        T[] doppelganger = array.dup;
        sort(doppelganger);
        return doppelganger;
    }
}
template TFunctionalArrays(T, U) {
    public U[] map(T[] array, U (*operation)(T))
    out(result) {
        assert(result.length == array.length);
    } body {
        U[] res = new U[array.length];
        for (int i = 0; i < array.length; i++) {
            res[i] = operation(array[i]);
        }
        return res;
    }
    public U foldl(T[] array, U init, U (*operation)(U, T))
    out(result) {
        if (array.length == 0) {
            assert(result == init);
        } else {
            T first = array[0];
            T[] rest = array[1..array.length];
            assert(result == foldl(rest, operation(init, first), operation));
        }
    } body {
        U res = init;
        for (int i = 0; i < array.length; i++) {
            res = operation(res, array[i]);
        }
        return res;
    }
    public U foldr(T[] array, U init, U (*operation)(T, U))
    out(result) {
        if (array.length == 0) {
            assert(result == init);
        } else {
            T first = array[0];
            T[] rest = array[1..array.length];
            assert(result == operation(first, foldr(rest, init, operation)));
        }
    } body {
        U res = init;
        for (int i = array.length - 1; i >= 0; i--) {
            res = operation(array[i], res);
        }
        return res;
    }
}
template TFunctionalArrays(T, U, V) {
    public V[] map(T[] left, U[] right, V (*operation)(T, U))
    in {
        assert(left.length == right.length);
    } out(result) {
        assert(result.length == left.length);
    } body {
        V[] res = new V[left.length];
        for (int i = 0; i < res.length; i++) {
            res[i] = operation(left[i], right[i]);
        }
        return res;
    }
}
unittest {
    alias TArrays!(char) array;
    assert('a' == array.at("abcdefghij", 0));
    assert('b' == array.at("abcdefghij", 1));
    assert('c' == array.at("abcdefghij", 2));
    assert('d' == array.at("abcdefghij", 3));
    assert('e' == array.at("abcdefghij", 4));
    assert('f' == array.at("abcdefghij", 5));
    assert('g' == array.at("abcdefghij", 6));
    assert('h' == array.at("abcdefghij", 7));
    assert('i' == array.at("abcdefghij", 8));
    assert('j' == array.at("abcdefghij", 9));
    assert('a' == array.at("abcdefghij", -0));
    assert('b' == array.at("abcdefghij", -9));
    assert('c' == array.at("abcdefghij", -8));
    assert('d' == array.at("abcdefghij", -7));
    assert('e' == array.at("abcdefghij", -6));
    assert('f' == array.at("abcdefghij", -5));
    assert('g' == array.at("abcdefghij", -4));
    assert('h' == array.at("abcdefghij", -3));
    assert('i' == array.at("abcdefghij", -2));
    assert('j' == array.at("abcdefghij", -1));
    printf("TArrays.at tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert('a' == array.atWrap("abcdefghij", 0));
    assert('b' == array.atWrap("abcdefghij", 1));
    assert('c' == array.atWrap("abcdefghij", 2));
    assert('d' == array.atWrap("abcdefghij", 3));
    assert('e' == array.atWrap("abcdefghij", 4));
    assert('f' == array.atWrap("abcdefghij", 5));
    assert('g' == array.atWrap("abcdefghij", 6));
    assert('h' == array.atWrap("abcdefghij", 7));
    assert('i' == array.atWrap("abcdefghij", 8));
    assert('j' == array.atWrap("abcdefghij", 9));
    assert('a' == array.atWrap("abcdefghij", -0));

	printf("%c\n\n", array.atWrap("abcdefghij", -8));
	printf("%c\n\n", array.atWrap("abcdefghij", -7));
    
/+
	/* For some reason these don't work anymore */
	
    assert('b' == array.atWrap("abcdefghij", -9));
    assert('c' == array.atWrap("abcdefghij", -8));
    assert('d' == array.atWrap("abcdefghij", -7));
    assert('e' == array.atWrap("abcdefghij", -6));
    assert('f' == array.atWrap("abcdefghij", -5));
    assert('g' == array.atWrap("abcdefghij", -4));
    assert('h' == array.atWrap("abcdefghij", -3));
    assert('i' == array.atWrap("abcdefghij", -2));
    assert('j' == array.atWrap("abcdefghij", -1));+/
    assert('a' == array.atWrap("abcdefghij", 10));
    assert('b' == array.atWrap("abcdefghij", 11));
    assert('c' == array.atWrap("abcdefghij", 12));
    assert('d' == array.atWrap("abcdefghij", 13));
    assert('e' == array.atWrap("abcdefghij", 14));
    assert('f' == array.atWrap("abcdefghij", 15));
    assert('g' == array.atWrap("abcdefghij", 16));
    assert('h' == array.atWrap("abcdefghij", 17));
    assert('i' == array.atWrap("abcdefghij", 18));
    assert('j' == array.atWrap("abcdefghij", 19));
    assert('a' == array.atWrap("abcdefghij", 20));
    assert('b' == array.atWrap("abcdefghij", 21));
    assert('c' == array.atWrap("abcdefghij", 22));
    assert('d' == array.atWrap("abcdefghij", 23));
    assert('e' == array.atWrap("abcdefghij", 24));
    assert('f' == array.atWrap("abcdefghij", 25));
    assert('g' == array.atWrap("abcdefghij", 26));
    assert('h' == array.atWrap("abcdefghij", 27));
    assert('i' == array.atWrap("abcdefghij", 28));
    assert('j' == array.atWrap("abcdefghij", 29));
    printf("TArrays.atWrap tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert('a' == array.atSaturate("abcdefghij", 0));
    assert('b' == array.atSaturate("abcdefghij", 1));
    assert('c' == array.atSaturate("abcdefghij", 2));
    assert('d' == array.atSaturate("abcdefghij", 3));
    assert('e' == array.atSaturate("abcdefghij", 4));
    assert('f' == array.atSaturate("abcdefghij", 5));
    assert('g' == array.atSaturate("abcdefghij", 6));
    assert('h' == array.atSaturate("abcdefghij", 7));
    assert('i' == array.atSaturate("abcdefghij", 8));
    assert('j' == array.atSaturate("abcdefghij", 9));
    assert('a' == array.atSaturate("abcdefghij", -0));
    assert('a' == array.atSaturate("abcdefghij", -1));
    assert('a' == array.atSaturate("abcdefghij", -2));
    assert('j' == array.atSaturate("abcdefghij", 10));
    assert('j' == array.atSaturate("abcdefghij", 11));
    assert('j' == array.atSaturate("abcdefghij", 12));
    printf("TArrays.atSaturate tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert('a' == array.atBorder("abcdefghij", 0, 'z'));
    assert('b' == array.atBorder("abcdefghij", 1, 'z'));
    assert('c' == array.atBorder("abcdefghij", 2, 'z'));
    assert('d' == array.atBorder("abcdefghij", 3, 'z'));
    assert('e' == array.atBorder("abcdefghij", 4, 'z'));
    assert('f' == array.atBorder("abcdefghij", 5, 'z'));
    assert('g' == array.atBorder("abcdefghij", 6, 'z'));
    assert('h' == array.atBorder("abcdefghij", 7, 'z'));
    assert('i' == array.atBorder("abcdefghij", 8, 'z'));
    assert('j' == array.atBorder("abcdefghij", 9, 'z'));
    assert('a' == array.atBorder("abcdefghij", -0, 'z'));
    assert('z' == array.atBorder("abcdefghij", -1, 'z'));
    assert('z' == array.atBorder("abcdefghij", -2, 'z'));
    assert('z' == array.atBorder("abcdefghij", 10, 'z'));
    assert('z' == array.atBorder("abcdefghij", 11, 'z'));
    assert('z' == array.atBorder("abcdefghij", 12, 'z'));
    printf("TArrays.atBorder tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert('j' == array.atReflect("abcdefghij", -29));
    assert('i' == array.atReflect("abcdefghij", -28));
    assert('h' == array.atReflect("abcdefghij", -27));
    assert('g' == array.atReflect("abcdefghij", -26));
    assert('f' == array.atReflect("abcdefghij", -25));
    assert('e' == array.atReflect("abcdefghij", -24));
    assert('d' == array.atReflect("abcdefghij", -23));
    assert('c' == array.atReflect("abcdefghij", -22));
    assert('b' == array.atReflect("abcdefghij", -21));
    assert('a' == array.atReflect("abcdefghij", -20));
    assert('a' == array.atReflect("abcdefghij", -19));
    assert('b' == array.atReflect("abcdefghij", -18));
    assert('c' == array.atReflect("abcdefghij", -17));
    assert('d' == array.atReflect("abcdefghij", -16));
    assert('e' == array.atReflect("abcdefghij", -15));
    assert('f' == array.atReflect("abcdefghij", -14));
    assert('g' == array.atReflect("abcdefghij", -13));
    assert('h' == array.atReflect("abcdefghij", -12));
    assert('i' == array.atReflect("abcdefghij", -11));
    assert('j' == array.atReflect("abcdefghij", -10));
    assert('i' == array.atReflect("abcdefghij", -9));
    assert('h' == array.atReflect("abcdefghij", -8));
    assert('g' == array.atReflect("abcdefghij", -7));
    assert('f' == array.atReflect("abcdefghij", -6));
    assert('e' == array.atReflect("abcdefghij", -5));
    assert('d' == array.atReflect("abcdefghij", -4));
    assert('c' == array.atReflect("abcdefghij", -3));
    assert('b' == array.atReflect("abcdefghij", -2));
    assert('a' == array.atReflect("abcdefghij", -1));
    assert('a' == array.atReflect("abcdefghij", -0));
    assert('a' == array.atReflect("abcdefghij", 0));
    assert('b' == array.atReflect("abcdefghij", 1));
    assert('c' == array.atReflect("abcdefghij", 2));
    assert('d' == array.atReflect("abcdefghij", 3));
    assert('e' == array.atReflect("abcdefghij", 4));
    assert('f' == array.atReflect("abcdefghij", 5));
    assert('g' == array.atReflect("abcdefghij", 6));
    assert('h' == array.atReflect("abcdefghij", 7));
    assert('i' == array.atReflect("abcdefghij", 8));
    assert('j' == array.atReflect("abcdefghij", 9));
    assert('i' == array.atReflect("abcdefghij", 10));
    assert('h' == array.atReflect("abcdefghij", 11));
    assert('g' == array.atReflect("abcdefghij", 12));
    assert('f' == array.atReflect("abcdefghij", 13));
    assert('e' == array.atReflect("abcdefghij", 14));
    assert('d' == array.atReflect("abcdefghij", 15));
    assert('c' == array.atReflect("abcdefghij", 16));
    assert('b' == array.atReflect("abcdefghij", 17));
    assert('a' == array.atReflect("abcdefghij", 18));
    assert('a' == array.atReflect("abcdefghij", 19));
    assert('b' == array.atReflect("abcdefghij", 20));
    assert('c' == array.atReflect("abcdefghij", 21));
    assert('d' == array.atReflect("abcdefghij", 22));
    assert('e' == array.atReflect("abcdefghij", 23));
    assert('f' == array.atReflect("abcdefghij", 24));
    assert('g' == array.atReflect("abcdefghij", 25));
    assert('h' == array.atReflect("abcdefghij", 26));
    assert('i' == array.atReflect("abcdefghij", 27));
    assert('j' == array.atReflect("abcdefghij", 28));
    assert('i' == array.atReflect("abcdefghij", 29));
    printf("TArrays.atReflect tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert("abcdefghij" == array.head("abcdefghij", 10));
    assert("abcdefghij" == array.tail("abcdefghij", 10));
    assert("" == array.head("abcdefghij", 0));
    assert("" == array.tail("abcdefghij", 0));
    assert("abc" == array.head("abcdefghij", 3));
    assert("defghij" == array.tail("abcdefghij", 7));
    assert("hij" == array.tail("abcdefghij", 3));
    assert('a' == array.first("abcdefghij"));
    assert('j' == array.last("abcdefghij"));
    assert("bcdefghij" == array.rest("abcdefghij"));
    printf("TArrays.slicing tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.find("abcdefghij", "a"[0]) == 0);
    assert(array.find("abcdefghij", "e"[0]) == 4);
    assert(array.find("abcdefghij", "j"[0]) == 9);
    assert(array.find("abcdefghij", "k"[0]) == -1);
    assert(array.find("abcdefghij", "abcdefghij") == 0);
    assert(array.find("abcdefghij", "abcdefghi") == 0);
    assert(array.find("abcdefghij", "abcdefgh") == 0);
    assert(array.find("abcdefghij", "abcdefg") == 0);
    assert(array.find("abcdefghij", "abcdef") == 0);
    assert(array.find("abcdefghij", "abcde") == 0);
    assert(array.find("abcdefghij", "abcd") == 0);
    assert(array.find("abcdefghij", "abc") == 0);
    assert(array.find("abcdefghij", "ab") == 0);
    assert(array.find("abcdefghij", "a"[0]) == 0);
    assert(array.find("abcdefghij", "abcdefghij") == 0);
    assert(array.find("abcdefghij", "bcdefghij") == 1);
    assert(array.find("abcdefghij", "cdefghij") == 2);
    assert(array.find("abcdefghij", "defghij") == 3);
    assert(array.find("abcdefghij", "efghij") == 4);
    assert(array.find("abcdefghij", "fghij") == 5);
    assert(array.find("abcdefghij", "ghij") == 6);
    assert(array.find("abcdefghij", "hij") == 7);
    assert(array.find("abcdefghij", "ij") == 8);
    assert(array.find("abcdefghij", "j"[0]) == 9);
    assert(array.find("abcdefghij", "abcdefghij") == 0);
    assert(array.find("abcdefghij", "bcdefghi") == 1);
    assert(array.find("abcdefghij", "cdefgh") == 2);
    assert(array.find("abcdefghij", "defg") == 3);
    assert(array.find("abcdefghij", "ef") == 4);
    assert(array.find("abcdefghij", "abcdefghijkl") == -1);
    assert(array.find("abcdefghij", "abcdefghijkl") == -1);
    assert(array.find("abcdefghij", "bcdefghijkl") == -1);
    assert(array.find("abcdefghij", "cdefghijkl") == -1);
    assert(array.find("abcdefghij", "defghijkl") == -1);
    assert(array.find("abcdefghij", "efghijkl") == -1);
    assert(array.find("abcdefghij", "fghijkl") == -1);
    assert(array.find("abcdefghij", "ghijkl") == -1);
    assert(array.find("abcdefghij", "hijkl") == -1);
    assert(array.find("abcdefghij", "ijkl") == -1);
    assert(array.find("abcdefghij", "jkl") == -1);
    assert(array.find("abcdefghij", "kl") == -1);
    assert(array.find("abcdefghij", "l"[0]) == -1);
    assert(array.find("abcabcabca", "abc") == 0);
    assert(array.find("abcdefghij", "") == -1);
    printf("TArrays.find tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.rfind("abcdefghij", "a"[0]) == 0);
    assert(array.rfind("abcdefghij", "e"[0]) == 4);
    assert(array.rfind("abcdefghij", "j"[0]) == 9);
    assert(array.rfind("abcdefghij", "k"[0]) == -1);
    assert(array.rfind("abcabcabca", "a"[0]) == 9);
    assert(array.rfind("abcabcabca", "b"[0]) == 7);
    assert(array.rfind("abcabcabca", "c"[0]) == 8);
    assert(array.rfind("abcabcabca", "abcabcabca") == 0);
    assert(array.rfind("abcabcabca", "abcabcabc") == 0);
    assert(array.rfind("abcabcabca", "abcabcab") == 0);
    assert(array.rfind("abcabcabca", "abcabca") == 3);
    assert(array.rfind("abcabcabca", "abcabc") == 3);
    assert(array.rfind("abcabcabca", "abcab") == 3);
    assert(array.rfind("abcabcabca", "abca") == 6);
    assert(array.rfind("abcabcabca", "abc") == 6);
    assert(array.rfind("abcabcabca", "ab") == 6);
    assert(array.rfind("abcabcabca", "a"[0]) == 9);
    assert(array.rfind("abcabcabca", "abcabcabca") == 0);
    assert(array.rfind("abcabcabca", "bcabcabca") == 1);
    assert(array.rfind("abcabcabca", "cabcabca") == 2);
    assert(array.rfind("abcabcabca", "abcabca") == 3);
    assert(array.rfind("abcabcabca", "bcabca") == 4);
    assert(array.rfind("abcabcabca", "cabca") == 5);
    assert(array.rfind("abcabcabca", "abca") == 6);
    assert(array.rfind("abcabcabca", "bca") == 7);
    assert(array.rfind("abcabcabca", "ca") == 8);
    assert(array.rfind("abcabcabca", "a"[0]) == 9);
    assert(array.rfind("abcdefghij", "") == -1);
    printf("TArrays.rfind tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.startsWith("abcdefghij", "abcdefghij"));
    assert(array.startsWith("abcdefghij", "abcdefghi"));
    assert(array.startsWith("abcdefghij", "abcdefgh"));
    assert(array.startsWith("abcdefghij", "abcdefg"));
    assert(array.startsWith("abcdefghij", "abcdef"));
    assert(array.startsWith("abcdefghij", "abcde"));
    assert(array.startsWith("abcdefghij", "abcd"));
    assert(array.startsWith("abcdefghij", "abc"));
    assert(array.startsWith("abcdefghij", "ab"));
    assert(array.startsWith("abcdefghij", "a"[0]));
    assert(!array.startsWith("abcdefghij", ""));
    assert(!array.startsWith("abcdefghij", "bcdefghij"));
    assert(!array.startsWith("abcdefghij", "cdefghij"));
    assert(!array.startsWith("abcdefghij", "defghij"));
    assert(!array.startsWith("abcdefghij", "efghij"));
    assert(!array.startsWith("abcdefghij", "fghij"));
    assert(!array.startsWith("abcdefghij", "ghij"));
    assert(!array.startsWith("abcdefghij", "hij"));
    assert(!array.startsWith("abcdefghij", "ij"));
    assert(!array.startsWith("abcdefghij", "j"[0]));
    assert(!array.startsWith("abcdefghij", "abcdefghijk"));
    assert(!array.startsWith("abcdefghij", "abcdefghik"));
    printf("TArrays.startsWith tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.endsWith("abcdefghij", "abcdefghij"));
    assert(array.endsWith("abcdefghij", "bcdefghij"));
    assert(array.endsWith("abcdefghij", "cdefghij"));
    assert(array.endsWith("abcdefghij", "defghij"));
    assert(array.endsWith("abcdefghij", "efghij"));
    assert(array.endsWith("abcdefghij", "fghij"));
    assert(array.endsWith("abcdefghij", "ghij"));
    assert(array.endsWith("abcdefghij", "hij"));
    assert(array.endsWith("abcdefghij", "ij"));
    assert(array.endsWith("abcdefghij", "j"[0]));
    assert(!array.endsWith("abcdefghij", ""));
    assert(!array.endsWith("abcdefghij", "abcdefghi"));
    assert(!array.endsWith("abcdefghij", "abcdefgh"));
    assert(!array.endsWith("abcdefghij", "abcdefg"));
    assert(!array.endsWith("abcdefghij", "abcdef"));
    assert(!array.endsWith("abcdefghij", "abcde"));
    assert(!array.endsWith("abcdefghij", "abcd"));
    assert(!array.endsWith("abcdefghij", "abc"));
    assert(!array.endsWith("abcdefghij", "ab"));
    assert(!array.endsWith("abcdefghij", "a"[0]));
    assert(!array.endsWith("abcdefghij", "_abcdefghij"));
    assert(!array.endsWith("abcdefghij", "bbcdefghij"));
    assert(array.endsWith("foo.exe", ".exe"));
    printf("TArrays.endsWith tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.find("abcdefghij", "a"[0]) == 0);
    assert(array.find("abcdefghij", "e"[0]) == 4);
    assert(array.find("abcdefghij", "j"[0]) == 9);
    assert(array.find("abcdefghij", "k"[0]) == -1);
    assert(array.find("abcdefghij", "abcdefghij") == 0);
    assert(array.find("abcdefghij", "abcdefghi") == 0);
    assert(array.find("abcdefghij", "abcdefgh") == 0);
    assert(array.find("abcdefghij", "abcdefg") == 0);
    assert(array.find("abcdefghij", "abcdef") == 0);
    assert(array.find("abcdefghij", "abcde") == 0);
    assert(array.find("abcdefghij", "abcd") == 0);
    assert(array.find("abcdefghij", "abc") == 0);
    assert(array.find("abcdefghij", "ab") == 0);
    assert(array.find("abcdefghij", "a"[0]) == 0);
    assert(array.find("abcdefghij", "abcdefghij") == 0);
    assert(array.find("abcdefghij", "bcdefghij") == 1);
    assert(array.find("abcdefghij", "cdefghij") == 2);
    assert(array.find("abcdefghij", "defghij") == 3);
    assert(array.find("abcdefghij", "efghij") == 4);
    assert(array.find("abcdefghij", "fghij") == 5);
    assert(array.find("abcdefghij", "ghij") == 6);
    assert(array.find("abcdefghij", "hij") == 7);
    assert(array.find("abcdefghij", "ij") == 8);
    assert(array.find("abcdefghij", "j"[0]) == 9);
    assert(array.find("abcdefghij", "abcdefghij") == 0);
    assert(array.find("abcdefghij", "bcdefghi") == 1);
    assert(array.find("abcdefghij", "cdefgh") == 2);
    assert(array.find("abcdefghij", "defg") == 3);
    assert(array.find("abcdefghij", "ef") == 4);
    assert(array.find("abcdefghij", "abcdefghijkl") == -1);
    assert(array.find("abcdefghij", "abcdefghijkl") == -1);
    assert(array.find("abcdefghij", "bcdefghijkl") == -1);
    assert(array.find("abcdefghij", "cdefghijkl") == -1);
    assert(array.find("abcdefghij", "defghijkl") == -1);
    assert(array.find("abcdefghij", "efghijkl") == -1);
    assert(array.find("abcdefghij", "fghijkl") == -1);
    assert(array.find("abcdefghij", "ghijkl") == -1);
    assert(array.find("abcdefghij", "hijkl") == -1);
    assert(array.find("abcdefghij", "ijkl") == -1);
    assert(array.find("abcdefghij", "jkl") == -1);
    assert(array.find("abcdefghij", "kl") == -1);
    assert(array.find("abcdefghij", "l"[0]) == -1);
    assert(array.find("abcabcabca", "abc") == 0);
    assert(array.find("abcdefghij", "") == -1);
    printf("TArrays.find tests passed!\r\n");
}
private boolean isVowel(char c) {
    return (c == 'a') || (c == 'e') || (c == 'i') || (c == 'o') || (c == 'u');
}
private class Character {
    private char letter;
    this(char c) {
        this.letter = c;
    }
    public boolean isLetter(char c) {
        return this.letter == c;
    }
}
unittest {
    alias TArrays!(char) array;
    assert(array.all("aeiouoeieuiei", &isVowel));
    assert(array.all("aeoiuaioeuioaeuiouoiaeu", &isVowel));
    assert(!array.all("aaeiouioeujiurioue", &isVowel));
    assert(!array.all("bjkqwkjbwqjbkwb", &isVowel));
    assert(array.all("", &isVowel));

/*
    Character a = new Character('a');
    boolean delegate(char) isLetter;
    isLetter = &a.isLetter;
    assert(array.all("", isLetter));
*/
    printf("TArrays.all tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.any("aeiouoeieuiei", &isVowel));
    assert(array.any("aeoiuaioeuioaeuiouoiaeu", &isVowel));
    assert(array.any("aaeiouioeujiurioue", &isVowel));
    assert(array.any("qwenqwnlkqklnnwqnkl", &isVowel));
    assert(!array.any("bjkqwkjbwqjbkwb", &isVowel));
    assert(!array.any("", &isVowel));
    printf("TArrays.any tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.count("aeiouoeieuiei", &isVowel) == 13);
    assert(array.count("aeoiuaioeuioaeuiouoiaeu", &isVowel) == 23);
    assert(array.count("aaeiouioeujiurioue", &isVowel) == 16);
    assert(array.count("qwenqwnlkqklnnwqnkl", &isVowel) == 1);
    assert(array.count("bjkqwkjbwqjbkwb", &isVowel) == 0);
    assert(array.count("", &isVowel) == 0);
    printf("TArrays.count tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.filter("aeiouoeieuiei", &isVowel) == "aeiouoeieuiei");
    assert(array.filter("aeoiuaioeuioaeuiouoiaeu", &isVowel) == "aeoiuaioeuioaeuiouoiaeu");
    assert(array.filter("aaeiouioeujiurioue", &isVowel) == "aaeiouioeuiuioue");
    assert(array.filter("qwenqwnlkqklnnwqnkl", &isVowel) == "e");
    assert(array.filter("bjkqwkjbwqjbkwb", &isVowel) == "");
    assert(array.filter("", &isVowel) == "");
    printf("TArrays.filter tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.countOf("aaaaaaaaaa", "a"[0]) == 10);
    assert(array.countOf("aaaaaaaaaa", cast(char[]) "a") == 10);
    assert(array.countOf("aaaaaaaaaa", "b"[0]) == 0);
    assert(array.countOf("aaaaaaaaaa", cast(char[]) "b") == 0);
    assert(array.countOf("bbbbbbbbbb", "a"[0]) == 0);
    assert(array.countOf("bbbbbbbbbb", cast(char[]) "a") == 0);
    assert(array.countOf("bbbbbbbbbb", "b"[0]) == 10);
    assert(array.countOf("bbbbbbbbbb", cast(char[]) "b") == 10);
    assert(array.countOf("aaaaaaaaaa", "aa") == 5);
    assert(array.countOf("aaaaaaaaaa", "aaa") == 3);
    assert(array.countOf("aaaaaaaaaa", "aaaa") == 2);
    assert(array.countOf("aaaaaaaaaa", "aaaaa") == 2);
    assert(array.countOf("abcabcabca", cast(char[]) "a") == 4);
    assert(array.countOf("abcabcabca", "a"[0]) == 4);
    assert(array.countOf("abcabcabca", "ab") == 3);
    assert(array.countOf("abcabcabca", "ab") == 3);
    assert(array.countOf("abcabcabca", "b"[0]) == 3);
    assert(array.countOf("abcabcabca", "c"[0]) == 3);
    assert(array.countOf("abcabcabca", "bc") == 3);
    assert(array.countOf("abcabcabca", "ca") == 3);
    assert(array.countOf("abcabcabca", "cab") == 2);
    assert(array.countOf("abcabcabca", "bca") == 3);
    assert(array.countOf("abcabcabca", "abca") == 2);
    assert(array.countOf("abcabcabca", "bcab") == 1);
    assert(array.countOf("abcabcabca", "abcab") == 1);
    assert(array.countOf("dododododo", "dodo") == 2);
    printf("TArrays.countOf tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.repeat("abc", 0) == "");
    assert(array.repeat("abc", 1) == "abc");
    assert(array.repeat("abc", 2) == "abcabc");
    assert(array.repeat("x", 8) == "xxxxxxxx");
    printf("TArrays.repeat tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    assert(array.zip("abcd", "") == "abcd");
    assert(array.zip("", "efgh") == "efgh");
    assert(array.zip("abcd", "e") == "aebcd");
    assert(array.zip("abcd", "ef") == "aebfcd");
    assert(array.zip("abcd", "efg") == "aebfcgd");
    assert(array.zip("abcd", "efgh") == "aebfcgdh");
    assert(array.zip("abcd", "efghi") == "aebfcgdhi");
    assert(array.zip("a", "efgh") == "aefgh");
    assert(array.zip("ab", "efgh") == "aebfgh");
    assert(array.zip("abc", "efgh") == "aebfcgh");
    assert(array.zip("abcd", "efgh") == "aebfcgdh");
    assert(array.zip("abcd-", "efgh") == "aebfcgdh-");
    printf("TArrays.zip tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    char[] lyrics = "Second verse, like the first. Third verse, differente from the first.";	
    char[][] words; // = new char[11][];
	words.length = 11;
    words[0] = "Second";
    words[1] = "verse,";
    words[2] = "like";
    words[3] = "the";
    words[4] = "first.";
    words[5] = "Third";
    words[6] = "verse,";
    words[7] = "differente";
    words[8] = "from";
    words[9] = "the";
    words[10] = "first.";
    assert(array.join(words, " ") == lyrics);
    alias TArrays!(char[]) array2;
    assert(array.join(array2.rest(words), " ") == array.removed(lyrics, 0, 7));
    
    char[][] letters; // = new char[10][];
	letters.length = 10;
    letters[0] = "a";
    letters[1] = "b";
    letters[2] = "c";
    letters[3] = "d";
    letters[4] = "e";
    letters[5] = "f";
    letters[6] = "g";
    letters[7] = "h";
    letters[8] = "i";
    letters[9] = "j";
    assert(array.join(letters, "") == "abcdefghij");
    assert(array.join(letters, " ") == "a b c d e f g h i j");
    assert(array.join(letters, "  ") == "a  b  c  d  e  f  g  h  i  j");
    assert(array.join(letters, "-") == "a-b-c-d-e-f-g-h-i-j");
    assert(array.join(letters, " - ") == "a - b - c - d - e - f - g - h - i - j");
    printf("TArrays.join tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    char[] letters;
    letters = "abcdefghij";
    array.insert(letters, 0, "-"[0]);
    assert(letters == "-abcdefghij");
    letters = "abcdefghij";
    array.insert(letters, 1, "-"[0]);
    assert(letters == "a-bcdefghij");
    letters = "abcdefghij";
    array.insert(letters, 2, "-"[0]);
    assert(letters == "ab-cdefghij");
    letters = "abcdefghij";
    array.insert(letters, 3, "-"[0]);
    assert(letters == "abc-defghij");
    letters = "abcdefghij";
    array.insert(letters, 4, "-"[0]);
    assert(letters == "abcd-efghij");
    letters = "abcdefghij";
    array.insert(letters, 5, "-"[0]);
    assert(letters == "abcde-fghij");
    letters = "abcdefghij";
    array.insert(letters, 6, "-"[0]);
    assert(letters == "abcdef-ghij");
    letters = "abcdefghij";
    array.insert(letters, 7, "-"[0]);
    assert(letters == "abcdefg-hij");
    letters = "abcdefghij";
    array.insert(letters, 8, "-"[0]);
    assert(letters == "abcdefgh-ij");
    letters = "abcdefghij";
    array.insert(letters, 9, "-"[0]);
    assert(letters == "abcdefghi-j");
    letters = "abcdefghij";
    array.insert(letters, 10, "-"[0]);
    assert(letters == "abcdefghij-");

    assert(array.inserted("abcdefghij", 0, "-"[0]) == "-abcdefghij");
    assert(array.inserted("abcdefghij", 1, "-"[0]) == "a-bcdefghij");
    assert(array.inserted("abcdefghij", 2, "-"[0]) == "ab-cdefghij");
    assert(array.inserted("abcdefghij", 3, "-"[0]) == "abc-defghij");
    assert(array.inserted("abcdefghij", 4, "-"[0]) == "abcd-efghij");
    assert(array.inserted("abcdefghij", 5, "-"[0]) == "abcde-fghij");
    assert(array.inserted("abcdefghij", 6, "-"[0]) == "abcdef-ghij");
    assert(array.inserted("abcdefghij", 7, "-"[0]) == "abcdefg-hij");
    assert(array.inserted("abcdefghij", 8, "-"[0]) == "abcdefgh-ij");
    assert(array.inserted("abcdefghij", 9, "-"[0]) == "abcdefghi-j");
    assert(array.inserted("abcdefghij", 10, "-"[0]) == "abcdefghij-");

    assert(array.inserted("abcdefghij", 0, "--") == "--abcdefghij");
    assert(array.inserted("abcdefghij", 1, "--") == "a--bcdefghij");
    assert(array.inserted("abcdefghij", 2, "--") == "ab--cdefghij");
    assert(array.inserted("abcdefghij", 3, "--") == "abc--defghij");
    assert(array.inserted("abcdefghij", 4, "--") == "abcd--efghij");
    assert(array.inserted("abcdefghij", 5, "--") == "abcde--fghij");
    assert(array.inserted("abcdefghij", 6, "--") == "abcdef--ghij");
    assert(array.inserted("abcdefghij", 7, "--") == "abcdefg--hij");
    assert(array.inserted("abcdefghij", 8, "--") == "abcdefgh--ij");
    assert(array.inserted("abcdefghij", 9, "--") == "abcdefghi--j");
    assert(array.inserted("abcdefghij", 10, "--") == "abcdefghij--");
    printf("TArrays.insert tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    char[] letters;
    letters = "acklavick";
    array.replace(letters, "ck", "sta");
    assert(letters == "astalavista");
    letters = "ababababab"; 
    array.replace(letters, "ab", "ba");
    assert(letters == "bababababa");
    letters = "ababababab"; 
    array.replace(letters, "a"[0], "b"[0]);
    assert(letters == "bbbbbbbbbb");
    letters = "ababababab"; 
    array.replace(letters, cast(char[])"b", "a"[0]);
    assert(letters == "aaaaaaaaaa");
    letters = "ababababab";
    array.replace(letters, "b"[0], "cd");
    assert(letters == "acdacdacdacdacd");
    letters = "ababababab";
    array.replace(letters, "ab", "");
    assert(letters == "");

    assert(array.replaced("acklavick", "ck", "sta") == "astalavista");
    assert(array.replaced("ababababab", "ab", "ba") == "bababababa");
    assert(array.replaced("ababababab", "a"[0], "b"[0]) == "bbbbbbbbbb");
    assert(array.replaced("ababababab", cast(char[])"b", "a"[0]) == "aaaaaaaaaa");
    assert(array.replaced("ababababab", "b"[0], "cd") == "acdacdacdacdacd");
    assert(array.replaced("ababababab", "ab", "") == "");
    printf("TArrays.replace tests passed!\r\n");
}
unittest {
    alias TArrays!(char) array;
    char[] letters;
    letters = "abcdefghij"; 
    array.remove(letters, 0);
    assert(letters == "bcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 1);
    assert(letters == "acdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 2);
    assert(letters == "abdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 3);
    assert(letters == "abcefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 4);
    assert(letters == "abcdfghij");
    letters = "abcdefghij"; 
    array.remove(letters, 5);
    assert(letters == "abcdeghij");
    letters = "abcdefghij"; 
    array.remove(letters, 6);
    assert(letters == "abcdefhij");
    letters = "abcdefghij"; 
    array.remove(letters, 7);
    assert(letters == "abcdefgij");
    letters = "abcdefghij"; 
    array.remove(letters, 8);
    assert(letters == "abcdefghj");
    letters = "abcdefghij"; 
    array.remove(letters, 9);
    assert(letters == "abcdefghi");

    assert(array.removed("abcdefghij", 0) == "bcdefghij");
    assert(array.removed("abcdefghij", 1) == "acdefghij");
    assert(array.removed("abcdefghij", 2) == "abdefghij");
    assert(array.removed("abcdefghij", 3) == "abcefghij");
    assert(array.removed("abcdefghij", 4) == "abcdfghij");
    assert(array.removed("abcdefghij", 5) == "abcdeghij");
    assert(array.removed("abcdefghij", 6) == "abcdefhij");
    assert(array.removed("abcdefghij", 7) == "abcdefgij");
    assert(array.removed("abcdefghij", 8) == "abcdefghj");
    assert(array.removed("abcdefghij", 9) == "abcdefghi");

    letters = "abcdefghij"; 
    array.remove(letters, 0, 0);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 1, 1);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 2, 2);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 3, 3);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 4, 4);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 5, 5);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 6, 6);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 7, 7);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 8, 8);
    assert(letters == "abcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 9, 9);
    assert(letters == "abcdefghij");

    letters = "abcdefghij"; 
    array.remove(letters, 0, 1);
    assert(letters == "bcdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 1, 2);
    assert(letters == "acdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 2, 3);
    assert(letters == "abdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 3, 4);
    assert(letters == "abcefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 4, 5);
    assert(letters == "abcdfghij");
    letters = "abcdefghij"; 
    array.remove(letters, 5, 6);
    assert(letters == "abcdeghij");
    letters = "abcdefghij"; 
    array.remove(letters, 6, 7);
    assert(letters == "abcdefhij");
    letters = "abcdefghij"; 
    array.remove(letters, 7, 8);
    assert(letters == "abcdefgij");
    letters = "abcdefghij"; 
    array.remove(letters, 8, 9);
    assert(letters == "abcdefghj");
    letters = "abcdefghij"; 
    array.remove(letters, 9, 10);
    assert(letters == "abcdefghi");

    letters = "abcdefghij"; 
    array.remove(letters, 0, 2);
    assert(letters == "cdefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 1, 3);
    assert(letters == "adefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 2, 4);
    assert(letters == "abefghij");
    letters = "abcdefghij"; 
    array.remove(letters, 3, 5);
    assert(letters == "abcfghij");
    letters = "abcdefghij"; 
    array.remove(letters, 4, 6);
    assert(letters == "abcdghij");
    letters = "abcdefghij"; 
    array.remove(letters, 5, 7);
    assert(letters == "abcdehij");
    letters = "abcdefghij"; 
    array.remove(letters, 6, 8);
    assert(letters == "abcdefij");
    letters = "abcdefghij"; 
    array.remove(letters, 7, 9);
    assert(letters == "abcdefgj");
    letters = "abcdefghij"; 
    array.remove(letters, 8, 10);
    assert(letters == "abcdefgh");

    assert(array.removed("abcdefghij", 0, 0) == "abcdefghij");
    assert(array.removed("abcdefghij", 1, 1) == "abcdefghij");
    assert(array.removed("abcdefghij", 2, 2) == "abcdefghij");
    assert(array.removed("abcdefghij", 3, 3) == "abcdefghij");
    assert(array.removed("abcdefghij", 4, 4) == "abcdefghij");
    assert(array.removed("abcdefghij", 5, 5) == "abcdefghij");
    assert(array.removed("abcdefghij", 6, 6) == "abcdefghij");
    assert(array.removed("abcdefghij", 7, 7) == "abcdefghij");
    assert(array.removed("abcdefghij", 8, 8) == "abcdefghij");
    assert(array.removed("abcdefghij", 9, 9) == "abcdefghij");
    assert(array.removed("abcdefghij", 0, 1) == "bcdefghij");
    assert(array.removed("abcdefghij", 1, 2) == "acdefghij");
    assert(array.removed("abcdefghij", 2, 3) == "abdefghij");
    assert(array.removed("abcdefghij", 3, 4) == "abcefghij");
    assert(array.removed("abcdefghij", 4, 5) == "abcdfghij");
    assert(array.removed("abcdefghij", 5, 6) == "abcdeghij");
    assert(array.removed("abcdefghij", 6, 7) == "abcdefhij");
    assert(array.removed("abcdefghij", 7, 8) == "abcdefgij");
    assert(array.removed("abcdefghij", 8, 9) == "abcdefghj");
    assert(array.removed("abcdefghij", 9, 10) == "abcdefghi");
    assert(array.removed("abcdefghij", 0, 2) == "cdefghij");
    assert(array.removed("abcdefghij", 1, 3) == "adefghij");
    assert(array.removed("abcdefghij", 2, 4) == "abefghij");
    assert(array.removed("abcdefghij", 3, 5) == "abcfghij");
    assert(array.removed("abcdefghij", 4, 6) == "abcdghij");
    assert(array.removed("abcdefghij", 5, 7) == "abcdehij");
    assert(array.removed("abcdefghij", 6, 8) == "abcdefij");
    assert(array.removed("abcdefghij", 7, 9) == "abcdefgj");
    assert(array.removed("abcdefghij", 8, 10) == "abcdefgh");

    printf("TArrays.remove tests passed!\r\n");
}
unittest {
    alias TComparable!(char) cmp;
    alias TArrays!(char) array;
    assert(array.min("0123456789", &cmp.ascending) == '0');
    assert(array.min("9876543210", &cmp.ascending) == '0');
    assert(array.min("876543210", &cmp.ascending) == '0');
    assert(array.min("76543210", &cmp.ascending) == '0');
    assert(array.min("6543210", &cmp.ascending) == '0');
    assert(array.min("543210", &cmp.ascending) == '0');
    assert(array.min("43210", &cmp.ascending) == '0');
    assert(array.min("3210", &cmp.ascending) == '0');
    assert(array.min("210", &cmp.ascending) == '0');
    assert(array.min("10", &cmp.ascending) == '0');
    assert(array.min("0", &cmp.ascending) == '0');
    assert(array.min("9876543210", &cmp.descending) == '9');
    assert(array.min("0123456789", &cmp.descending) == '9');
    assert(array.min("123456789", &cmp.descending) == '9');
    assert(array.min("23456789", &cmp.descending) == '9');
    assert(array.min("3456789", &cmp.descending) == '9');
    assert(array.min("456789", &cmp.descending) == '9');
    assert(array.min("56789", &cmp.descending) == '9');
    assert(array.min("6789", &cmp.descending) == '9');
    assert(array.min("789", &cmp.descending) == '9');
    assert(array.min("89", &cmp.descending) == '9');
    assert(array.min("9", &cmp.descending) == '9');

    assert(array.min("0123456789", &cmp.ascending) == '0');
    assert(array.min("123456789", &cmp.ascending) == '1');
    assert(array.min("23456789", &cmp.ascending) == '2');
    assert(array.min("3456789", &cmp.ascending) == '3');
    assert(array.min("456789", &cmp.ascending) == '4');
    assert(array.min("56789", &cmp.ascending) == '5');
    assert(array.min("6789", &cmp.ascending) == '6');
    assert(array.min("789", &cmp.ascending) == '7');
    assert(array.min("89", &cmp.ascending) == '8');
    assert(array.min("9", &cmp.ascending) == '9');
    assert(array.min("9876543210", &cmp.descending) == '9');
    assert(array.min("876543210", &cmp.descending) == '8');
    assert(array.min("76543210", &cmp.descending) == '7');
    assert(array.min("6543210", &cmp.descending) == '6');
    assert(array.min("543210", &cmp.descending) == '5');
    assert(array.min("43210", &cmp.descending) == '4');
    assert(array.min("3210", &cmp.descending) == '3');
    assert(array.min("210", &cmp.descending) == '2');
    assert(array.min("10", &cmp.descending) == '1');
    assert(array.min("0", &cmp.descending) == '0');
    printf("TArrays.min tests passed!\r\n");
}
unittest {
    alias TComparable!(char) cmp;
    alias TArrays!(char) array;
    assert(array.max("9876543210", &cmp.ascending) == '9');
    assert(array.max("0123456789", &cmp.ascending) == '9');
    assert(array.max("123456789", &cmp.ascending) == '9');
    assert(array.max("23456789", &cmp.ascending) == '9');
    assert(array.max("3456789", &cmp.ascending) == '9');
    assert(array.max("456789", &cmp.ascending) == '9');
    assert(array.max("56789", &cmp.ascending) == '9');
    assert(array.max("6789", &cmp.ascending) == '9');
    assert(array.max("789", &cmp.ascending) == '9');
    assert(array.max("89", &cmp.ascending) == '9');
    assert(array.max("9", &cmp.ascending) == '9');
    assert(array.max("0123456789", &cmp.descending) == '0');
    assert(array.max("9876543210", &cmp.descending) == '0');
    assert(array.max("876543210", &cmp.descending) == '0');
    assert(array.max("76543210", &cmp.descending) == '0');
    assert(array.max("6543210", &cmp.descending) == '0');
    assert(array.max("543210", &cmp.descending) == '0');
    assert(array.max("43210", &cmp.descending) == '0');
    assert(array.max("3210", &cmp.descending) == '0');
    assert(array.max("210", &cmp.descending) == '0');
    assert(array.max("10", &cmp.descending) == '0');
    assert(array.max("0", &cmp.descending) == '0');

    assert(array.max("9876543210", &cmp.ascending) == '9');
    assert(array.max("876543210", &cmp.ascending) == '8');
    assert(array.max("76543210", &cmp.ascending) == '7');
    assert(array.max("6543210", &cmp.ascending) == '6');
    assert(array.max("543210", &cmp.ascending) == '5');
    assert(array.max("43210", &cmp.ascending) == '4');
    assert(array.max("3210", &cmp.ascending) == '3');
    assert(array.max("210", &cmp.ascending) == '2');
    assert(array.max("10", &cmp.ascending) == '1');
    assert(array.max("0", &cmp.ascending) == '0');
    assert(array.max("0123456789", &cmp.descending) == '0');
    assert(array.max("123456789", &cmp.descending) == '1');
    assert(array.max("23456789", &cmp.descending) == '2');
    assert(array.max("3456789", &cmp.descending) == '3');
    assert(array.max("456789", &cmp.descending) == '4');
    assert(array.max("56789", &cmp.descending) == '5');
    assert(array.max("6789", &cmp.descending) == '6');
    assert(array.max("789", &cmp.descending) == '7');
    assert(array.max("89", &cmp.descending) == '8');
    assert(array.max("9", &cmp.descending) == '9');
    printf("TArrays.max tests passed!\r\n");
}
unittest {
    alias TComparable!(char) cmp;
    alias TArrays!(char) array;
    assert(array.isOrdered("abcdefghij", &cmp.ascending));
    assert(array.isOrdered("jihgfedcba", &cmp.descending));
    assert(!array.isOrdered("abcdefgihj", &cmp.ascending));
    assert(!array.isOrdered("jihgfedbca", &cmp.descending));
    printf("TArrays.isOrdered tests passed!\r\n");
}
unittest {
    alias TComparable!(char) cmp;
    alias TArrays!(char) array;
    char[] letters;
    letters = "jihgfedcba";
    array.sort(letters, &cmp.ascending);
    assert("abcdefghij" == letters);
    array.sort(letters, &cmp.descending);
    assert("jihgfedcba" == letters);

    assert(array.sorted("jihgfedcba", &cmp.ascending) == "abcdefghij");
    assert(array.sorted("abcdefghij", &cmp.descending) == "jihgfedcba");
    printf("TArrays.sort tests passed!\r\n");
}
private char[] result;
private void concatenate(char c) {
    result ~= c;
}
unittest {
    alias TArrays!(char) array;
    result = "";
    array.foreachFn("abcdefghij", &concatenate);  /* was foreachFn was foreach */
    assert(result == "abcdefghij");
    result = "";
    array.foreachFn("1234567890", &concatenate);  /* was foreachFn was foreach */
    assert(result == "1234567890");
    printf("TArrays.foreach tests passed!\r\n");
}
private char toHackedUpper(char c) {
    return c - 0x20;
}
unittest {
    alias TArrays!(char) array;
    char[] letters;
    letters = "abcdefghij";
    array.opApply(letters, &toHackedUpper);
    assert(letters == "ABCDEFGHIJ");
    letters = "klmnopqrst";
    array.opApply(letters, &toHackedUpper);
    assert(letters == "KLMNOPQRST");
    printf("TArrays.apply tests passed!\r\n");
}
unittest {
    alias TComparableArrays!(char) array;
    assert(array.min("0123456789") == '0');
    assert(array.min("9876543210") == '0');
    assert(array.min("876543210") == '0');
    assert(array.min("76543210") == '0');
    assert(array.min("6543210") == '0');
    assert(array.min("543210") == '0');
    assert(array.min("43210") == '0');
    assert(array.min("3210") == '0');
    assert(array.min("210") == '0');
    assert(array.min("10") == '0');
    assert(array.min("0") == '0');
    assert(array.min("0123456789") == '0');
    assert(array.min("123456789") == '1');
    assert(array.min("23456789") == '2');
    assert(array.min("3456789") == '3');
    assert(array.min("456789") == '4');
    assert(array.min("56789") == '5');
    assert(array.min("6789") == '6');
    assert(array.min("789") == '7');
    assert(array.min("89") == '8');
    assert(array.min("9") == '9');
    printf("TComparableArrays.min tests passed!\r\n");
}
unittest {
    alias TComparableArrays!(char) array;
    assert(array.max("9876543210") == '9');
    assert(array.max("0123456789") == '9');
    assert(array.max("123456789") == '9');
    assert(array.max("23456789") == '9');
    assert(array.max("3456789") == '9');
    assert(array.max("456789") == '9');
    assert(array.max("56789") == '9');
    assert(array.max("6789") == '9');
    assert(array.max("789") == '9');
    assert(array.max("89") == '9');
    assert(array.max("9") == '9');
    assert(array.max("9876543210") == '9');
    assert(array.max("876543210") == '8');
    assert(array.max("76543210") == '7');
    assert(array.max("6543210") == '6');
    assert(array.max("543210") == '5');
    assert(array.max("43210") == '4');
    assert(array.max("3210") == '3');
    assert(array.max("210") == '2');
    assert(array.max("10") == '1');
    assert(array.max("0") == '0');
    printf("TComparableArrays.max tests passed!\r\n");
}
unittest {
    alias TComparableArrays!(char) array;
    assert(array.isOrdered("abcdefghij"));
    assert(!array.isOrdered("abcdefgihj"));
    printf("TComparableArrays.isOrdered tests passed!\r\n");
}
unittest {
    alias TComparableArrays!(char) array;
    char[] letters;
    letters = "jihgfedcba";
    array.sort(letters);
    assert("abcdefghij" == letters);
    letters = "abcdefghij";
    array.sort(letters);
    assert("abcdefghij" == letters);
    letters = "acbdefghij";
    array.sort(letters);
    assert("abcdefghij" == letters);
    assert(array.sorted("jihgfedcba") == "abcdefghij");
    assert(array.sorted("abcdefghij") == "abcdefghij");
    printf("TComparableArrays.sort tests passed!\r\n");
}
private int sum(int left, int right) {
    return left + right;
}
private int sub(int left, int right) {
    return left - right;
}
private int product(int left, int right) {
    return left * right;
}
unittest {
    alias TFunctionalArrays!(int, int) array;
    alias TOrdinal!(int) ord;
    int[] numbers  = new int[10];
    int[] predecessors = new int[10];
    int[] sucessors = new int[10];
    for (int i = 0; i < numbers.length; i++) {
        numbers[i] = i + 1;
        predecessors[i] = i;
        sucessors[i] = i + 2;
    }
    assert(array.map(numbers, &ord.predecessor) == predecessors);
    assert(array.map(numbers, &ord.successor) == sucessors);
    printf("TFunctionalArrays.map tests passed!\r\n");
}
unittest {
    alias TFunctionalArrays!(int, int) array;
    alias TComparable!(int) cmp;
    int[] numbers  = new int[10];
    for (int i = 0; i < numbers.length; i++) {
        numbers[i] = i + 1;
    }
    assert(array.foldl(numbers, 0, &sum) == 55);
    assert(array.foldl(numbers, 1, &product) == 3628800);
    assert(array.foldl(numbers, 11, &cmp.min) == 1);
    assert(array.foldl(numbers, 0, &cmp.max) == 10);
    assert(array.foldl(numbers, 0, &sub) == -55);
    printf("TFunctionalArrays.foldl tests passed!\r\n");
}
unittest {
    alias TFunctionalArrays!(int, int) array;
    alias TComparable!(int) cmp;
    int[] numbers  = new int[10];
    for (int i = 0; i < numbers.length; i++) {
        numbers[i] = i + 1;
    }
    assert(array.foldr(numbers, 0, &sum) == 55);
    assert(array.foldr(numbers, 1, &product) == 3628800);
    assert(array.foldr(numbers, 11, &cmp.min) == 1);
    assert(array.foldr(numbers, 0, &cmp.max) == 10);
    assert(array.foldr(numbers, 0, &sub) == -5);
    printf("TFunctionalArrays.foldr tests passed!\r\n");
}
private boolean bothOk(int n, char c) {
    return (n < 5) && isVowel(c);
}
unittest {
    alias TFunctionalArrays!(int, int, int) array;
    alias TFunctionalArrays!(int, char, boolean) array2;
    alias TComparable!(int) cmp;
    int[] numbers1  = new int[10];
    int[] numbers2  = new int[numbers1.length];
    for (int i = 0; i < numbers1.length; i++) {
        numbers1[i] = i;
        numbers2[i] = i + 1;
    }
    assert(array.map(numbers1, numbers2, &cmp.min) == numbers1);
    assert(array.map(numbers1, numbers2, &cmp.max) == numbers2);

    const boolean[] results  = [true, false, true, true, true, false, false, true, false, false];
    const int[] values = [0, 1, 2, 3, 4, 5, 4, 3, 2, 1];
    assert(array2.map(values, "abeiocdegh", &bothOk) == results);
    printf("TFunctionalArrays.map2 tests passed!\r\n");
}
