/*
Collection of template array functions
Copyright (C) 2002  Daniel Yukio Yokomiso

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

See attached file lesser.txt for a copy of the GNU Lesser General Public License
 */

module deimos.base;
public alias int boolean;
public alias int Order;
public const Order LESS_THAN    = -1;
public const Order EQUALS_TO    = 0;
public const Order GREATER_THAN = +1;
template TComparable(T) {
    public T min(T left, T right) {
        return left < right ? left: right;
    }
    public T max(T left, T right) {
        return left > right ? left: right;
    }
    public T clamp(T item, T lower, T upper)
    in {
        assert(lower <= upper);
    } body {
        return max(min(item, upper), lower);
    }
    public Order ascending(T left, T right) {
        return (left < right) ? LESS_THAN
                              : ((left == right) ? EQUALS_TO
                                                 : GREATER_THAN);
    }
    public Order descending(char left, char right) {
        return (left > right) ? LESS_THAN
                              : ((left == right) ? EQUALS_TO
                                                 : GREATER_THAN);
    }
}
unittest {
    alias TComparable!(char) cmp;
    assert(cmp.clamp('a', 'b', 'e') == 'b');
    assert(cmp.clamp('b', 'b', 'e') == 'b');
    assert(cmp.clamp('c', 'b', 'e') == 'c');
    assert(cmp.clamp('d', 'b', 'e') == 'd');
    assert(cmp.clamp('e', 'b', 'e') == 'e');
    assert(cmp.clamp('f', 'b', 'e') == 'e');
    assert(cmp.min('a', 'b') == 'a');
    assert(cmp.min('a', 'c') == 'a');
    assert(cmp.min('a', 'd') == 'a');
    assert(cmp.min('a', 'e') == 'a');
    assert(cmp.max('a', 'b') == 'b');
    assert(cmp.max('a', 'c') == 'c');
    assert(cmp.max('a', 'd') == 'd');
    assert(cmp.max('a', 'e') == 'e');
    printf("TComparable(char) tests passed!\r\n");
}
unittest {
    alias TComparable!(int) cmp;
    assert(cmp.clamp(1, 2, 5) == 2);
    assert(cmp.clamp(2, 2, 5) == 2);
    assert(cmp.clamp(3, 2, 5) == 3);
    assert(cmp.clamp(4, 2, 5) == 4);
    assert(cmp.clamp(5, 2, 5) == 5);
    assert(cmp.clamp(6, 2, 5) == 5);
    assert(cmp.min(1, 2) == 1);
    assert(cmp.min(1, 3) == 1);
    assert(cmp.min(1, 4) == 1);
    assert(cmp.min(1, 5) == 1);
    assert(cmp.max(1, 2) == 2);
    assert(cmp.max(1, 3) == 3);
    assert(cmp.max(1, 4) == 4);
    assert(cmp.max(1, 5) == 5);
    printf("TComparable(int) tests passed!\r\n");
}
template TOrdinal(T) {
    alias TComparable!(T) tcomparable;
    public T predecessor(T item) {
        return item - 1;
    }
    public T successor(T item) {
        return item + 1;
    }
}
unittest {
    alias TOrdinal!(char) ord;
    assert(ord.predecessor('b') == 'a');
    assert(ord.predecessor('c') == 'b');
    assert(ord.predecessor('d') == 'c');
    assert(ord.successor('a') == 'b');
    assert(ord.successor('b') == 'c');
    assert(ord.successor('c') == 'd');
    printf("TOrdinal(char) tests passed!\r\n");
}
unittest {
    alias TOrdinal!(int) ord;
    assert(ord.predecessor(2) == 1);
    assert(ord.predecessor(3) == 2);
    assert(ord.predecessor(4) == 3);
    assert(ord.successor(1) == 2);
    assert(ord.successor(2) == 3);
    assert(ord.successor(3) == 4);
    printf("TOrdinal(int) tests passed!\r\n");
}

template TPair(T, U) {
    public class Pair {
        private T _left;
        private U _right;
        public this(T left, U right) {
            this._left = left;
            this._right = right;
        }
        public T left() {
            return this._left;
        }
        public U right() {
            return this._right;
        }
        public boolean opEquals(Object obj) {
            Pair other = cast(Pair) obj;
            if (other !== null) {
                return (left() == other.left()) && (right() == other.right());
            } else {
                return false;
            }
        }
    }
}
unittest {
    alias TPair!(char, char) tpair;
    tpair.Pair pairA = new tpair.Pair('a', 'b');
    tpair.Pair pairB = new tpair.Pair('a', 'b');
    tpair.Pair pairC = new tpair.Pair('c', 'd');
    assert('a' == pairA.left());
    assert('b' == pairA.right());
    assert('a' == pairB.left());
    assert('b' == pairB.right());
    assert('c' == pairC.left());
    assert('d' == pairC.right());
    assert(pairA == pairB);
    assert(pairA != pairC);
    assert(pairB != pairC);
    printf("TPair(char, char) tests passed!\r\n");
}
unittest {
    alias TPair!(int, float) tpair;
    tpair.Pair pairA = new tpair.Pair(1, 3.0);
    tpair.Pair pairB = new tpair.Pair(1, 3.0);
    tpair.Pair pairC = new tpair.Pair(4, 2.0);
    assert(1 == pairA.left());
    assert(3.0 === pairA.right());
    assert(1 == pairB.left());
    assert(3.0 == pairB.right());
    assert(4 == pairC.left());
    assert(2.0 == pairC.right());
    assert(pairA == pairB);
    assert(pairA != pairC);
    assert(pairB != pairC);
    printf("TPair(int, float) tests passed!\r\n");
}
