/*
Collection of template array functions
Copyright (C) 2002  Daniel Yukio Yokomiso

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

See attached file lesser.txt for a copy of the GNU Lesser General Public License
 */

module deimos.numerics;
import deimos.base;
import deimos.numerics;
import std.math;
template TIntegral(T) {
    public boolean divides(T left, T right) {
        return (right == 0) || (right % left) == 0;
    }
    public T gcd(T left, T right)
    in {
        assert(left >= 0);
        assert(right >= 0);
    } out(result) {
        assert((left == 0) || (left % result) == 0);
        assert((right == 0) || ((right % result) == 0));
    } body {
        if (right == 0) {
            return left;
        } else {
            return gcd(right, left % right);
        }
// uncomment following lines if you need a higher-perfomance non-recursive gcd
//        T x = left;
//        T y = right;
//        while (y != 0) {
//            T temp = y;
//            y = x % y;
//            x = temp;
//        }
//        return x;
    }
}
public class Fraction {
    private alias TIntegral!(long) integral;
// TODO uncomment following line when compiler bug is smashed
//    private instance TComparable(long) comp;
    private final long _numerator;
    private final long _denominator;
    public this(long numerator, long denominator)
    in {
        assert(denominator != 0);
    } body {
        // normalize while constructing
        boolean numeratorPositive = (numerator >= 0);
        boolean denominatorPositive = (denominator >= 0);
        long a = numeratorPositive ? numerator : -numerator;
        long b = denominatorPositive ? denominator : -denominator;
        long g = integral.gcd(a, b);
        this._numerator = (numeratorPositive == denominatorPositive)? (a / g) : (-a / g);
        this._denominator = b / g;
    }
    public this(Fraction other) {
        this._numerator = other.numerator();
        this._denominator = other.denominator();
    }
    public this(real other) {
        assert(false);// TODO when this is done include overloading of operators for extendeds
    }
    public final long numerator() {
        return this._numerator;
    }
    public final long denominator() {
        return this._denominator;
    }
    public boolean isNegative() {
        return numerator() < 0;
    }
    public char[] toString() {
        if (denominator() == 1) {
            return asString(numerator());
        } else {
            return asString(numerator()) ~ "/" ~ asString(denominator());
        }
    }
    public uint toHash() {
        return cast(uint) (this._numerator ^ this._denominator);
    }
    public double asDouble() { 
        return (cast(double) numerator()) / (cast(double) denominator());
    }
    public real  asReal() { /* was asExtended */
        return (cast(real) numerator()) / (cast(real) denominator());
    }
    public Fraction opNeg() {
        return new Fraction(-numerator(), denominator());
    }
    public Fraction opCom() /+inverse()+/ {
        return new Fraction(denominator(), numerator());
    }
    public Fraction opAdd(Fraction other) {
        return new Fraction(numerator() * other.denominator()
                          + other.numerator() * denominator(),
                           denominator() * other.denominator());
    }
    public Fraction opAdd(long other) {
        return new Fraction(numerator() + other * denominator(), denominator());
    }
    public Fraction opSub(Fraction other) {
        return new Fraction(numerator() * other.denominator()
                          - other.numerator() * denominator(),
                            denominator() * other.denominator());
    }
    public Fraction opSub(long other) {
        return new Fraction(numerator() - other * denominator(), denominator());
    }
    public Fraction opSub_r(long other) {
        return new Fraction(other * denominator() - numerator(), denominator());
    }
    public Fraction opMul(Fraction other) {
        return new Fraction(numerator() * other.numerator(),
                            denominator() * other.denominator());
    }
    public Fraction opMul(long other) {
        return new Fraction(numerator() * other, denominator());
    }
    public Fraction opDiv(Fraction other) {
        return new Fraction(numerator() * other.denominator(),
                            denominator() * other.numerator());
    }
    public Fraction opDiv(long other) {
        return new Fraction(numerator(), denominator() * other);
    }
    public Fraction opDiv_r(long other) {
        return new Fraction(other * denominator(), numerator());
    }
    public Fraction pow(long exponent) {
        if (exponent < 0) {      
            return new Fraction(cast(long) std.math.pow(cast(real) denominator(), cast(real) -exponent), /* jcc7: added casts. */
                                cast(long) std.math.pow(cast(real) numerator(), cast(real) -exponent)); /* jcc7: added casts. */
        } else {
            return new Fraction(cast(long) std.math.pow(cast(real) numerator(), cast(real) exponent), /* jcc7: added casts. */
                                cast(long) std.math.pow(cast(real) denominator(), cast(real) exponent)); /* jcc7: added casts. */
        }
    }
    public Order opCmp(Object obj) {
        Fraction other = cast(Fraction) obj;
        if (other !== null) {
            long x = numerator() * other.denominator();
            long y = other.numerator() * denominator();
// TODO uncomment following line when compiler bug is smashed
//            return comp.ascending(x, y);
            return (x < y) ? LESS_THAN
                           : ((x == y) ? EQUALS_TO
                                       : GREATER_THAN);
        } else {
            assert(false); // What should be the correct behavior?
        }
    }
    public Order opCmp(long other) {
        long x = numerator();
        long y = other * denominator();
// TODO uncomment following line when compiler bug is smashed
//        return comp.ascending(x, y);
        return (x < y) ? LESS_THAN
                       : ((x == y) ? EQUALS_TO
                                   : GREATER_THAN);
    }
    public boolean opEquals(Object obj) {
        Fraction other = cast(Fraction) obj;
        if (other !== null) {
            return opCmp(other) == EQUALS_TO;
        } else {
            return false;
        }
    }
    public boolean opEquals(long other) {
        return opCmp(other) == EQUALS_TO;
    }
}
private const static char[] charForDigit = "0123456789ABCDEF";
public char[] asString(long number) {
    char[] result;
    if (number < 0) {
        if (number == long.min) {
            return "-9223372036854775808";
        } else {
            result ~= '-';
            number = -number;
        }
    }
    if (number < 10L) { // 1 digit
        result ~= charForDigit[cast(int)number];
    } else if (number < 100L) { // 2 digits
        result ~= charForDigit[cast(int)(number / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 1000L) { // 3 digits
        result ~= charForDigit[cast(int)(number / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 10000L) { // 4 digits
        result ~= charForDigit[cast(int)(number / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 100000L) { // 5 digits
        result ~= charForDigit[cast(int)(number / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 1000000L) { // 6 digits
        result ~= charForDigit[cast(int)(number / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 10000000L) { // 7 digits
        result ~= charForDigit[cast(int)(number / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 100000000L) { // 8 digits
        result ~= charForDigit[cast(int)(number / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 1000000000L) { // 9 digits
        result ~= charForDigit[cast(int)(number / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 10000000000L) { // 10 digits
        result ~= charForDigit[cast(int)(number / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 100000000000L) { // 11 digits
        result ~= charForDigit[cast(int)(number / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 1000000000000L) { // 12 digits
        result ~= charForDigit[cast(int)(number / 100000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000L) / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 10000000000000L) { // 13 digits
        result ~= charForDigit[cast(int)(number / 1000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000L) / 100000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000L) / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 100000000000000L) { // 14 digits
        result ~= charForDigit[cast(int)(number / 10000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000L) / 1000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000L) / 100000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000L) / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 1000000000000000L) { // 15 digits
        result ~= charForDigit[cast(int)(number / 100000000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000000L) / 10000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000L) / 1000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000L) / 100000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000L) / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 10000000000000000L) { // 16 digits
        result ~= charForDigit[cast(int)(number / 1000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000000L) / 100000000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000000L) / 10000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000L) / 1000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000L) / 100000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000L) / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 100000000000000000L) { // 17 digits
        result ~= charForDigit[cast(int)(number / 10000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000000L) / 1000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000000L) / 100000000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000000L) / 10000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000L) / 1000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000L) / 100000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000L) / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else if (number < 1000000000000000000L) { // 18 digits
        result ~= charForDigit[cast(int)(number / 100000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000000000L) / 10000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000000L) / 1000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000000L) / 100000000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000000L) / 10000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000L) / 1000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000L) / 100000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000L) / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    } else { // 19 digits
        result ~= charForDigit[cast(int)(number / 1000000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000000000L) / 100000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000000000L) / 10000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000000L) / 1000000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000000L) / 100000000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000000L) / 10000000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000000L) / 1000000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000000L) / 100000000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000000L) / 10000000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000000L) / 1000000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000000L) / 100000000L)];
        result ~= charForDigit[cast(int)((number %= 100000000L) / 10000000L)];
        result ~= charForDigit[cast(int)((number %= 10000000L) / 1000000L)];
        result ~= charForDigit[cast(int)((number %= 1000000L) / 100000L)];
        result ~= charForDigit[cast(int)((number %= 100000L) / 10000L)];
        result ~= charForDigit[cast(int)((number %= 10000L) / 1000L)];
        result ~= charForDigit[cast(int)((number %= 1000L) / 100L)];
        result ~= charForDigit[cast(int)((number %= 100L) / 10L)];
        result ~= charForDigit[cast(int)(number % 10L)];
    }
}
unittest {
	
	/* For some reason these unittests are failing. */
	
	
	printf("Testing Fraction...\t");	
    Fraction n1 = new Fraction(1, 1);
	assert(n1.numerator() == 1);
    assert(n1.denominator() == 1);
    //printf("%d\n", n1.numerator());
    //printf("%d\n", n1.denominator());
  //  printf("%.*s\n", n1.toString());
//    assert(n1.toString() == "1"); /* For some unknown reason toString returns a "". */
    printf("a4");
//    assert(n1 == new Fraction(1, 1));
	
    printf("n2...");
    Fraction n2 = new Fraction(1, 2);
    assert(n2.numerator() == 1);
    assert(n2.denominator() == 2);
 //   assert(n2.toString() == "1/2");
    assert(n2 == new Fraction(1, 2));
    
	printf("n3...");
    Fraction n3 = new Fraction(2, 1);
    assert(n3.numerator() == 2);
    assert(n3.denominator() == 1);
 //   assert(n3.toString() == "2");
    assert(n3 == new Fraction(2, 1));
    assert(n1 != n2);
    assert(n1 != n3);
    assert(n2 != n3);
    assert(!(new Fraction(1,1)).isNegative());
    assert((new Fraction(-1,1)).isNegative());
    assert((new Fraction(1,-1)).isNegative());
    assert(!(new Fraction(-1,-1)).isNegative());
    assert(new Fraction(1, 2) == new Fraction(2, 4));
    assert(new Fraction(1, 2) == new Fraction(137, 274));
    assert((new Fraction(1249, 480)).toString() == "1249/480");
    assert((new Fraction(-1249, 480)).toString() == "-1249/480");
    assert((new Fraction(1249, -480)).toString() == "-1249/480");
    assert((new Fraction(-1249, -480)).toString() == "1249/480");
    printf("passed!\r\n");
}
unittest {
	assert(new Fraction(1, 1) + new Fraction(1,1) == new Fraction(2, 1));
    assert(new Fraction(1, 1) + 1 == new Fraction(2, 1));
    assert(1 + new Fraction(1, 1) == new Fraction(2, 1));
    assert(new Fraction(32, 15) + new Fraction(15, 32)
        == new Fraction(1249, 480));
    assert(new Fraction(32, 15) + 157 == new Fraction(2387, 15));
    assert(157 + new Fraction(32, 15) == new Fraction(2387, 15));
    assert(new Fraction(-32, 15) + new Fraction(-15, 32)
        == new Fraction(-1249, 480));
    assert(new Fraction(-32, 15) + 157 == new Fraction(2323, 15));
    assert(157 + new Fraction(-32, 15) == new Fraction(2323, 15));
    assert(new Fraction(32, -15) + 157 == new Fraction(2323, 15));
    assert(157 + new Fraction(32, -15) == new Fraction(2323, 15));
    assert(new Fraction(1, 1) + new Fraction(0, 1) == new Fraction(1, 1));
    assert(new Fraction(1, 1) - new Fraction(0, 1) == new Fraction(1, 1));
    assert(new Fraction(1, 1) - new Fraction(1, 1) == new Fraction(0, 1));
    assert(new Fraction(1, 1) + 1 == new Fraction(2, 1));
    assert(1 + new Fraction(1, 1) == new Fraction(2, 1));
    assert(new Fraction(32, 15) + new Fraction(15, 32)
        == new Fraction(1249, 480));
    assert(new Fraction(32, 15) + 157 == new Fraction(2387, 15));
    assert(157 + new Fraction(32, 15) == new Fraction(2387, 15));
    assert(new Fraction(-32, 15) + new Fraction(-15, 32)
        == new Fraction(-1249, 480));
    assert(new Fraction(-32, 15) + 157 == new Fraction(2323, 15));
    assert(157 + new Fraction(-32, 15) == new Fraction(2323, 15));
    assert(new Fraction(32, -15) + 157 == new Fraction(2323, 15));
    assert(157 + new Fraction(32, -15) == new Fraction(2323, 15));

    assert(new Fraction(32, 15) == new Fraction(-32, -15));
    assert(new Fraction(32, 15) == -new Fraction(-32, 15));
    assert(new Fraction(32, 15) == -new Fraction(32, -15));

    assert(new Fraction(1, 2) * new Fraction(1, 2) == new Fraction(1, 4));
    assert(new Fraction(2, 3) * new Fraction(2, 3) == new Fraction(4, 9));
    assert(new Fraction(2, 3) * new Fraction(3, 2) == new Fraction(1, 1));
    assert(new Fraction(1, 2) * new Fraction(2, 1) == new Fraction(1, 1));
    assert(new Fraction(1, 2) * 2 == new Fraction(1, 1));
    assert(2 * new Fraction(1, 2) == new Fraction(1, 1));
    assert(new Fraction(2, 3) * 2 == new Fraction(4, 3));
    assert(2 * new Fraction(2, 3) == new Fraction(4, 3));
    assert(new Fraction(-1, 2) * new Fraction(1, 2) == -new Fraction(1, 4));
    assert(new Fraction(2, 3) * new Fraction(-2, 3) == -new Fraction(4, 9));
    assert(new Fraction(-2, 3) * new Fraction(-3, 2) == new Fraction(1, 1));
    assert(-new Fraction(1, 2) * -new Fraction(2, 1) == new Fraction(1, 1));
    assert(new Fraction(1, 2) * -2 == -new Fraction(1, 1));
    assert(2 * -new Fraction(1, 2) == -new Fraction(1, 1));
    assert(-new Fraction(2, 3) * 2 == -new Fraction(4, 3));
    assert(-2 * new Fraction(2, 3) == -new Fraction(4, 3));

    assert(new Fraction(1, 2) / new Fraction(1, 2) == new Fraction(1, 1));
    assert(new Fraction(2, 3) / new Fraction(2, 3) == new Fraction(1, 1));
    assert(new Fraction(2, 3) / new Fraction(3, 2) == new Fraction(4, 9));
    assert(new Fraction(1, 2) / new Fraction(2, 1) == new Fraction(1, 4));
    assert(new Fraction(1, 2) / 2 == new Fraction(1, 4));
    assert(2 / new Fraction(1, 2) == new Fraction(4, 1));
    assert(new Fraction(2, 3) / 2 == new Fraction(1, 3));
    assert(2 / new Fraction(2, 3) == new Fraction(3, 1));
    assert(new Fraction(-1, 2) / new Fraction(1, 2) == -new Fraction(1, 1));
    assert(new Fraction(2, 3) / new Fraction(-2, 3) == -new Fraction(1, 1));
    assert(new Fraction(-2, 3) / new Fraction(-3, 2) == new Fraction(4, 9));
    assert(-new Fraction(1, 2) / -new Fraction(2, 1) == new Fraction(1, 4));
    assert(new Fraction(1, 2) / -2 == -new Fraction(1, 4));
    assert(2 / -new Fraction(1, 2) == -new Fraction(4, 1));
    assert(-new Fraction(2, 3) / 2 == -new Fraction(1, 3));
    assert(-2 / new Fraction(2, 3) == -new Fraction(3, 1));

    assert(new Fraction(1, 2) < 1);
    assert(new Fraction(2, 3) < 1);
    assert(new Fraction(2, 3) < new Fraction(3, 2));
    assert(new Fraction(2, 3) > new Fraction(4, 9));
    assert(new Fraction(3, 2) < new Fraction(9, 4));
    assert(new Fraction(1, 2) > new Fraction(1, 4));
    assert(new Fraction(2, 3) > new Fraction(1, 3));
    assert(new Fraction(-1, 2) < new Fraction(1, 2));
    assert(new Fraction(2, 3) > new Fraction(-2, 3));
    assert(new Fraction(-2, 3) > new Fraction(-3, 2));
    assert(-new Fraction(1, 2) > -new Fraction(2, 1));
    assert(-new Fraction(1, 4) > -new Fraction(1, 2));
    assert(-new Fraction(1, 3) > -new Fraction(2, 3));
    assert(-new Fraction(2, 3) > -3);

    assert((new Fraction(1, 1)).pow(0) == 1);
    assert((new Fraction(1, 1)).pow(1) == 1);
    assert((new Fraction(1, 1)).pow(2) == 1);
    assert((new Fraction(1, 1)).pow(3) == 1);
    assert((new Fraction(1, 2)).pow(0) == 1);
    assert((new Fraction(1, 2)).pow(1) == new Fraction(1, 2));
    assert((new Fraction(1, 2)).pow(2) == new Fraction(1, 4));
    assert((new Fraction(1, 2)).pow(3) == new Fraction(1, 8));
    assert((new Fraction(2, 3)).pow(0) == 1);
    assert((new Fraction(2, 3)).pow(1) == new Fraction(2, 3));
    assert((new Fraction(2, 3)).pow(2) == new Fraction(4, 9));
    assert((new Fraction(2, 3)).pow(3) == new Fraction(8, 27));
    assert((new Fraction(-1, 1)).pow(0) == 1);
    assert((new Fraction(-1, 1)).pow(1) == -1);
    assert((new Fraction(-1, 1)).pow(2) == 1);
    assert((new Fraction(-1, 1)).pow(3) == -1);
    assert((new Fraction(-1, 2)).pow(0) == 1);
    assert((new Fraction(-1, 2)).pow(1) == -new Fraction(1, 2));
    assert((new Fraction(-1, 2)).pow(2) == new Fraction(1, 4));
    assert((new Fraction(-1, 2)).pow(3) == -new Fraction(1, 8));
    assert((new Fraction(-2, 3)).pow(0) == 1);
    assert((new Fraction(-2, 3)).pow(1) == -new Fraction(2, 3));
    assert((new Fraction(-2, 3)).pow(2) == new Fraction(4, 9));
    assert((new Fraction(-2, 3)).pow(3) == -new Fraction(8, 27));
    assert((new Fraction(1, 1)).pow(-0) == 1);
    assert((new Fraction(1, 1)).pow(-1) == 1);
    assert((new Fraction(1, 1)).pow(-2) == 1);
    assert((new Fraction(1, 1)).pow(-3) == 1);
    assert((new Fraction(1, 2)).pow(-0) == 1);
    assert((new Fraction(1, 2)).pow(-1) == 2);
    assert((new Fraction(1, 2)).pow(-2) == 4);
    assert((new Fraction(1, 2)).pow(-3) == 8);
    assert((new Fraction(2, 3)).pow(-0) == 1);
    assert((new Fraction(2, 3)).pow(-1) == new Fraction(3, 2));
    assert((new Fraction(2, 3)).pow(-2) == new Fraction(9, 4));
    assert((new Fraction(2, 3)).pow(-3) == new Fraction(27, 8));
    assert((new Fraction(-1, 1)).pow(-0) == 1);
    assert((new Fraction(-1, 1)).pow(-1) == -1);
    assert((new Fraction(-1, 1)).pow(-2) == 1);
    assert((new Fraction(-1, 1)).pow(-3) == -1);
    assert((new Fraction(-1, 2)).pow(-0) == 1);
    assert((new Fraction(-1, 2)).pow(-1) == -2);
    assert((new Fraction(-1, 2)).pow(-2) == 4);
    assert((new Fraction(-1, 2)).pow(-3) == -8);
    assert((new Fraction(-2, 3)).pow(-0) == 1);
    assert((new Fraction(-2, 3)).pow(-1) == -new Fraction(3, 2));
    assert((new Fraction(-2, 3)).pow(-2) == new Fraction(9, 4));
    assert((new Fraction(-2, 3)).pow(-3) == -new Fraction(27, 8));
    printf("Fraction Arithmetic tests passed!\r\n");
}
unittest {
    alias TIntegral!(int) integral;
    assert(1 == integral.gcd(5, 2));
    assert(2 == integral.gcd(10, 2));
    assert(5 == integral.gcd(10, 5));
    assert(4 == integral.gcd(12, 8));
    printf("Integral.gcd tests passed!\r\n");
}
unittest {
    alias TIntegral!(int) integral;
    assert(!integral.divides(2, 5));
    assert(integral.divides(2, 10));
    assert(integral.divides(5, 10));
    assert(!integral.divides(8, 12));
    assert(integral.divides(4, 12));
    printf("Integral.divides tests passed!\r\n");
}
