/*
Collection of template array functions
Copyright (C) 2002  Daniel Yukio Yokomiso

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

See attached file lesser.txt for a copy of the GNU Lesser General Public License
 */
module deimos.range;
import deimos.base;
template TRange(T) {
    private alias TComparable!(T) comp;
    private boolean recursing = false;
    public class Range {
        private T _lower;
        private T _upper;
        public this(T lower, T upper) {
            this._lower = lower;
            this._upper = upper;
        }
        public T lower() {
            return this._lower;
        }
        public T upper() {
            return this._upper;
        }
        public boolean contains(T item) {
            return (lower() <= item) && (item <= upper());
        }
        public boolean intersects(Range other)
        out (result) {
            if (!recursing) {
                recursing = true;
                assert(result == other.intersects(this));
            } else {
                recursing = false;
            }
        } body {
            return contains(other.lower()) || contains(other.upper()) || other.includes(this);
        }
        public boolean includes(Range other)
        out (result) {
            assert(result == (contains(other.lower()) && contains(other.upper())));
        } body {
            return contains(other.lower()) && contains(other.upper());
        }
        public Range opSub_r(Range other) {
            return other.opSub(this);
        }
        public Range opSub(Range other)
        in {
            assert(intersects(other));
        } out (result) {
            assert(includes(result) && other.includes(result));
        } body {
            return new Range(comp.max(lower(), other.lower()),
                             comp.min(upper(), other.upper()));
        }
        public Range opAdd(Range other)
        out (result) {
            assert(result.includes(this) && result.includes(other));
        } body {
            return new Range(comp.min(lower(), other.lower()),
                             comp.max(upper(), other.upper()));
        }
        public boolean opEquals(Object obj) {
            Range other = cast(Range) obj;
            if (other !== null) {
                return (lower() == other.lower()) && (upper() == other.upper());
            } else {
                return false;
            }
        }
        invariant {
            assert(this._lower <= this._upper);
        }
    }
}
unittest {
	printf("Test TRange(char).Range... \t");    
    alias TRange!(char) trange;
	
    printf("any...\t");    
    trange.Range any = new trange.Range('a', 'z');
    assert('a' == any.lower());
    assert('z' == any.upper());
    assert(any.contains('a'));
    assert(any.contains('b'));
    assert(any.contains('c'));
    assert(any.contains('d'));
    assert(any.contains('w'));
    assert(any.contains('x'));
    assert(any.contains('y'));
    assert(any.contains('z'));
    assert(!any.contains('A'));
    assert(!any.contains('Z'));
    
	printf("other...\t");    
    trange.Range other = new trange.Range('X', 'c');
    assert(any.intersects(other));
    trange.Range another = new trange.Range('A', 'Z');
    assert(!any.intersects(another));
    assert(any.includes(new trange.Range('a', 'e')));
    assert(any.includes(new trange.Range('f', 'j')));
    assert(!any.includes(new trange.Range('Z', 'c')));
	assert(new trange.Range('a', 'e') - new trange.Range('c', 'g') == new trange.Range('c', 'e'));
    assert(new trange.Range('a', 'e') + new trange.Range('c', 'g') == new trange.Range('a', 'g'));
    assert((new trange.Range('a', 'j')).intersects(new trange.Range('d', 'f')));
    printf("passed!\r\n");
}
unittest {
    alias TRange!(int) trange;
    trange.Range any = new trange.Range(1, 10);
    assert(1 == any.lower());
    assert(10 == any.upper());
    assert(any.contains(1));
    assert(any.contains(2));
    assert(any.contains(3));
    assert(any.contains(4));
    assert(any.contains(5));
    assert(any.contains(6));
    assert(any.contains(7));
    assert(any.contains(8));
    assert(any.contains(9));
    assert(any.contains(10));
    assert(!any.contains(0));
    assert(!any.contains(11));
    trange.Range other = new trange.Range(-3, 3);
    assert(any.intersects(other));
    trange.Range another = new trange.Range(-10, 0);
    assert(!any.intersects(another));
    assert(any.includes(new trange.Range(1, 5)));
    assert(any.includes(new trange.Range(6, 10)));
    assert(!any.includes(new trange.Range(0, 3)));
    assert(new trange.Range(1, 5) - new trange.Range(3, 7) == new trange.Range(3, 5));
    assert(new trange.Range(1, 5) + new trange.Range(3, 7) == new trange.Range(1, 7));
    assert((new trange.Range(1, 10)).intersects(new trange.Range(4, 6)));
    printf("TRange(int).Range tests passed!\r\n");
}

template TDiscreteRange(T) {
    private alias TOrdinal!(T) ord;
    private boolean recursing = false;
    private alias TRange!(T) range;
    public boolean neighbours(range.Range left, range.Range right)
    out (result) {
        if (!recursing) {
            recursing = true;
            assert(result == neighbours(right, left));
        } else {
            recursing = false;
        }
    } body {
        return (left.lower() == ord.successor(right.upper()))
            || (ord.successor(left.upper()) == right.lower());
    }
    public int size(range.Range some)
    out(result) {
        assert(result > 0);
    } body {
        int res = 0;
        T lower = some.lower();
        while (lower <= some.upper()) {
            res++;
            lower = ord.successor(lower);
        }
        return res;
    } 
    public range.Range successor(range.Range some)
    out(result) {
        assert(result.lower() == ord.successor(some.upper()));
        assert(size(result) == size(some));
    } body {
        T start = ord.successor(some.upper());
        return new range.Range(start, start + size(some) - 1);
    }
    public range.Range predecessor(range.Range some)
    out(result) {
        assert(result.upper() == ord.predecessor(some.lower()));
        assert(size(result) == size(some));
    } body {
        T end = ord.predecessor(some.lower());
        return new range.Range(end - size(some) + 1, end);
    }
    public T[] toArray(range.Range some)
    out(result) {
        assert(result.length == size(some));
    } body {
        T[] res = new T[size(some)];
        T item = some.lower();
        for (int i = 0; i < res.length; i++) {
            res[i] = item;
            item = ord.successor(item);
        }
        return res;
    }
}
unittest {
	printf("Test TDiscreteRange(char)... \t");
    alias TRange!(char) trange;
    alias TDiscreteRange!(char) tdiscreterange;
    assert(tdiscreterange.neighbours(new trange.Range('a', 'e'), new trange.Range('f', 'j')));
    assert(!tdiscreterange.neighbours(new trange.Range('a', 'e'), new trange.Range('e', 'j')));
    assert(!tdiscreterange.neighbours(new trange.Range('a', 'e'), new trange.Range('g', 'j')));
    assert(tdiscreterange.size(new trange.Range('a', 'e')) == 5);
    assert(tdiscreterange.size(new trange.Range('a', 'z')) == 26);
    assert(tdiscreterange.size(new trange.Range('b', 'b')) == 1);
    assert(tdiscreterange.size(new trange.Range('c', 'g')) == 5);
    assert(tdiscreterange.toArray(new trange.Range('a', 'e')) == "abcde");
    assert(tdiscreterange.toArray(new trange.Range('a', 'z')) == "abcdefghijklmnopqrstuvwxyz");
    assert(tdiscreterange.toArray(new trange.Range('b', 'b')) == "b");
    assert(tdiscreterange.toArray(new trange.Range('c', 'g')) == "cdefg");
    assert(tdiscreterange.successor(new trange.Range('a', 'e')) == (new trange.Range('f', 'j')));
    assert(tdiscreterange.predecessor(new trange.Range('f', 'j')) == (new trange.Range('a', 'e')));
    assert((new trange.Range('a', 'j')).intersects(new trange.Range('d', 'f')));
    printf("TDiscreteRange(char) tests passed!\r\n");
}
unittest {
    alias TRange!(int) trange;
    alias TDiscreteRange!(int) tdiscreterange;
    assert(tdiscreterange.neighbours(new trange.Range(1, 5), new trange.Range(6, 10)));
    assert(!tdiscreterange.neighbours(new trange.Range(1, 5), new trange.Range(5, 10)));
    assert(!tdiscreterange.neighbours(new trange.Range(1, 5), new trange.Range(7, 10)));
    assert(tdiscreterange.size(new trange.Range(1, 5)) == 5);
    assert(tdiscreterange.size(new trange.Range(1, 10)) == 10);
    assert(tdiscreterange.size(new trange.Range(2, 2)) == 1);
    assert(tdiscreterange.size(new trange.Range(3, 7)) == 5);
    const int[5]  a1 = [1,2,3,4,5];
    const int[10] a2 = [1,2,3,4,5,6,7,8,9,10];
    const int[1]  a3 = [2];
    const int[5]  a4 = [3,4,5,6,7];
    assert(tdiscreterange.toArray(new trange.Range(1, 5)) == a1);
    assert(tdiscreterange.toArray(new trange.Range(1, 10)) == a2);
    assert(tdiscreterange.toArray(new trange.Range(2, 2)) == a3);
    assert(tdiscreterange.toArray(new trange.Range(3, 7)) == a4);
    assert(tdiscreterange.successor(new trange.Range(1, 5)) == (new trange.Range(6, 10)));
    assert(tdiscreterange.predecessor(new trange.Range(6, 10)) == (new trange.Range(1, 5)));
    printf("TDiscreteRange(int).Range tests passed!\r\n");
}
