/*
Collection of template array functions
Copyright (C) 2002  Daniel Yukio Yokomiso

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

See attached file lesser.txt for a copy of the GNU Lesser General Public License
 */

module deimos.rangeset;
import deimos.range;
import deimos.base;
import deimos.arrays;
template TRangeSet(T) {
    private alias TOrdinal!(T) ord;
    private alias TRange!(T) trange;
    private alias TDiscreteRange!(T) tdiscreterange;
    public alias trange.Range Range;
// TODO uncomment following line when compiler bug is smashed
//    private instance TArrays(Range) arrays;
    public class RangeSet {
        private Range[] items;
        public this(T[] members) {
            for (int i = 0; i < members.length; i++) {
                put(members[i]);
            }
        }
        public this() {
            this.items = new Range[0];
        }
        public void put(T lower, T upper)
        in {
            assert(lower <= upper);
        } out {
            assert(contains(lower));
            assert(contains(upper));
        } body {
            put(new Range(lower, upper));
        }
        public void put(Range range)
        out {
            assert(contains(range));
        } body {
            Range[] result = new Range[0];
            putIn(this.items, result, range);
            this.items = result;
        }
        public void put(T item)
        out {
            assert(contains(item));
        } body {
            put(item, item);
        }
        public void remove(T lower, T upper)
        in {
            assert(lower <= upper);
        } out {
            assert(!contains(lower));
            assert(!contains(upper));
        } body {
            remove(new Range(lower, upper));
        }
        public void remove(Range range)
        out {
            assert(!contains(range));
        } body {
            Range[] result = new Range[0];
            removeFrom(this.items, result, range);
            this.items = result;
        }
        public void remove(T item)
        out {
            assert(!contains(item));
        } body {
            remove(item, item);
        }
        public int size() {
            int result = 0;
            for (int i = 0; i < this.items.length; i++) {
                result += tdiscreterange.size(this.items[i]);
            }
            return result;
        }
        public boolean contains(Range range) {
            return contains(range.lower()) && contains(range.upper());
        }
        public boolean contains(T item) {
            return binaryContains(0, this.items.length, item);
        }
        public boolean binaryContains(int start, int afterEnd, char c) {
            if (start >= afterEnd) {
                return false;
            } else {
                int middle = start + ((afterEnd - start) / 2);
                if (this.items[middle].contains(c)) {
                    return true;
                } else if (this.items[middle].lower() > c) {
                    return binaryContains(start, middle, c);
                } else {
                    return binaryContains(middle + 1, afterEnd, c);
                }
            }
        }
        public boolean containsAll(T[] array) {
            for (int i = 0; i < array.length; i++) {
                if (!contains(array[i])) {
                    return false;
                }
            }
            return true;
        }
        public boolean isEmpty() {
            return size() == 0;
        }
    }
    private void putIn(Range[] source, inout Range[] result, Range range) {
        if (0 == source.length) {
            result ~= range;
        } else {
            Range first = source[0];
            Range[] rest = source[1 .. source.length];
//            Range first = arrays.first(source);
//            Range[] rest = arrays.rest(source);
            if (range.intersects(first) || tdiscreterange.neighbours(range, first)) {
                putIn(rest, result, range + first);
            } else if (range.upper() < first.lower()) {
                result ~= range;
                result ~= source;
            } else if (range.lower() > first.upper()) {
                result ~= first;
                putIn(rest, result, range);
            } else {
                assert(false);
            }
        }
    }
    private void removeFrom(Range[] source, inout Range[] result, Range range) {
        if (!(0 == source.length)) {
            Range first = source[0];
            Range[] rest = source[1 .. source.length];
//            Range first = arrays.first(source);
//            Range[] rest = arrays.rest(source);
            if (range.upper() < first.lower()) {
                result ~= source;
            } else if (range.lower() > first.upper()) {
                result ~= first;
                removeFrom(rest, result, range);
            } else if (range.includes(first)) {
                removeFrom(rest, result, range);
            } else if ((first.lower() < range.lower()) && (first.upper() < range.upper())) {
                result ~= new Range(first.lower(), ord.predecessor(range.lower()));
                removeFrom(rest, result, range);
            } else if ((range.lower() < first.lower()) && (range.upper() < first.upper())) {
                result ~= new Range(ord.successor(range.upper()), first.upper());
                result ~= rest;
            } else if ((first.lower() < range.lower()) && (range.upper() < first.upper())) {
                result ~= new Range(first.lower(), ord.predecessor(range.lower()));
                result ~= new Range(ord.successor(range.upper()), first.upper());
                result ~= rest;
            } else if (first.lower() < range.lower()) {
                result ~= new Range(first.lower(), ord.predecessor(range.lower()));
                result ~= rest;
            } else if (range.upper() < first.upper()) {
                result ~= new Range(ord.successor(range.upper()), first.upper());
                result ~= rest;
            } else {
                assert(0);
            }
        }
    }
}
unittest {
	printf("Test TRangeSet(char).new... \t");
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet("aeiouAEIOU");
    assert(ranges.containsAll("aeiouAEIOU"));
    ranges.remove('A');
    assert(ranges.containsAll("aeiouEIOU"));
    assert(!ranges.contains('A'));
    ranges.put('A');
    assert(ranges.containsAll("aeiouEIOU"));
    ranges.remove('e');
    assert(ranges.containsAll("aiouEIOU"));
    assert(!ranges.contains('e'));
    printf("TRangeSet(char).new from array passed!\r\n");
}
unittest {
    printf("Test TRangeSet(char).RangeSet... \t");
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    assert(0 == ranges.size());
    ranges.put('d', 'f');
    assert(ranges.containsAll("def"));
    assert(3 == ranges.size());
    ranges.put('a', 'c');
    assert(ranges.containsAll("abcdef"));
    assert(6 == ranges.size());
    ranges.put('h', 'j');
    assert(ranges.containsAll("abcdefhij"));
    assert(9 == ranges.size());
    assert(!ranges.contains('g'));
    ranges.put('g');
    assert(ranges.containsAll("abcdefghij"));
    assert(10 == ranges.size());
    ranges.put('a', 'c');
    assert(ranges.containsAll("abcdefghij"));
    assert(10 == ranges.size());
    ranges.put('d', 'f');
    assert(ranges.containsAll("abcdefghij"));
    assert(10 == ranges.size());
    ranges.put('h', 'j');
    assert(ranges.containsAll("abcdefghij"));
    assert(10 == ranges.size());
    ranges.put('g');
    assert(ranges.containsAll("abcdefghij"));
    assert(10 == ranges.size());
    ranges.put('a', 'f');
    assert(ranges.containsAll("abcdefghij"));
    assert(10 == ranges.size());
    ranges.put('d', 'h');
    assert(ranges.containsAll("abcdefghij"));
    assert(10 == ranges.size());
    ranges.put('f', 'j');
    assert(10 == ranges.size());
    assert(ranges.containsAll("abcdefghij"));
    ranges.put('a');
    assert(10 == ranges.size());
    assert(ranges.containsAll("abcdefghij"));
    ranges.put('j');
    assert(10 == ranges.size());
    assert(ranges.containsAll("abcdefghij"));
    printf("TRangeSet(char).RangeSet.put passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    assert(0 == ranges.size());
    ranges.put('a', 'j');
    assert(10 == ranges.size());
    assert(ranges.containsAll("abcdefghij"));
    ranges.remove('a', 'b');
    assert(ranges.containsAll("cdefghij"));
    assert(!ranges.contains('a'));
    assert(!ranges.contains('b'));
    assert(8 == ranges.size());
    ranges.remove('c', 'g');
    assert(ranges.containsAll("hij"));
    assert(!ranges.contains('c'));
    assert(!ranges.contains('d'));
    assert(!ranges.contains('e'));
    assert(!ranges.contains('f'));
    assert(!ranges.contains('g'));
    assert(3 == ranges.size());
    ranges.remove('h', 'j');
    assert(!ranges.contains('h'));
    assert(!ranges.contains('i'));
    assert(!ranges.contains('j'));
    assert(0 == ranges.size());
    ranges.put('a', 'j');
    assert(10 == ranges.size());
    ranges.remove('d', 'h');
    assert(ranges.containsAll("abcij"));
    assert(!ranges.contains('d'));
    assert(!ranges.contains('e'));
    assert(!ranges.contains('f'));
    assert(!ranges.contains('g'));
    assert(!ranges.contains('h'));
    assert(5 == ranges.size());
    printf("TRangeSet(char).RangeSet.remove tests passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    ranges.put('c', 'd');
    ranges.put('f', 'j');
    assert(ranges.containsAll("cdfghij"));
    ranges.remove('a', 'b');
    assert(ranges.containsAll("cdfghij"));
    printf("TRangeSet(char).RangeSet.removeRangeLowerThanFirst tests passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    ranges.put('c', 'd');
    ranges.put('f', 'j');
    assert(ranges.containsAll("cdfghij"));
    ranges.remove('e', 'f');
    assert(ranges.containsAll("cdghij"));
    printf("TRangeSet(char).RangeSet.removeRangeUpperThanFirst tests passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    ranges.put('b', 'c');
    ranges.put('e', 'j');
    assert(ranges.containsAll("bcefghij"));
    ranges.remove('a', 'd');
    assert(ranges.containsAll("efghij"));
    ranges = new charset.RangeSet();
    ranges.put('a', 'c');
    ranges.put('f', 'j');
    assert(ranges.containsAll("abcfghij"));
    ranges.remove('a', 'd');
    assert(ranges.containsAll("fghij"));
    ranges = new charset.RangeSet();
    ranges.put('b', 'd');
    ranges.put('f', 'j');
    assert(ranges.containsAll("bcdfghij"));
    ranges.remove('a', 'd');
    assert(ranges.containsAll("fghij"));
    printf("TRangeSet(char).RangeSet.removeRangeIncludingFirst tests passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    ranges.put('a', 'd');
    ranges.put('f', 'j');
    assert(ranges.containsAll("abcdfghij"));
    ranges.remove('a', 'd');
    assert(ranges.containsAll("fghij"));
    printf("TRangeSet(char).RangeSet.removeRangeEqualsToFirst tests passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    ranges.put('b', 'c');
    ranges.put('e', 'j');
    assert(ranges.containsAll("bcefghij"));
    ranges.remove('a', 'f');
    assert(ranges.containsAll("ghij"));
    printf("TRangeSet(char).RangeSet.removeRangeIncludingFirstAndPartOfSecond tests passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    ranges.put('a', 'd');
    ranges.put('f', 'j');
    assert(ranges.containsAll("abcdfghij"));
    ranges.remove('a', 'c');
    assert(ranges.containsAll("dfghij"));
    ranges = new charset.RangeSet();
    ranges.put('a', 'd');
    ranges.put('f', 'j');
    assert(ranges.containsAll("abcdfghij"));
    ranges.remove('b', 'd');
    assert(ranges.containsAll("afghij"));
    ranges = new charset.RangeSet();
    ranges.put('a', 'd');
    ranges.put('f', 'j');
    assert(ranges.containsAll("bcdfghij"));
    ranges.remove('b', 'c');
    assert(ranges.containsAll("adfghij"));
    printf("TRangeSet(char).RangeSet.removeRangeIncludedInFirst tests passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    ranges.put('b', 'd');
    ranges.put('f', 'j');
    assert(ranges.containsAll("bcdfghij"));
    ranges.remove('a', 'c');
    assert(ranges.containsAll("dfghij"));
    printf("TRangeSet(char).RangeSet.removeRangeIntersectingBeforeFirst tests passed!\r\n");
}
unittest {
    alias TRangeSet!(char) charset;
    charset.RangeSet ranges = new charset.RangeSet();
    ranges.put('a', 'c');
    ranges.put('e', 'j');
    assert(ranges.containsAll("abcefghij"));
    ranges.remove('b', 'd');
    assert(ranges.containsAll("aefghij"));
    printf("TRangeSet(char).RangeSet.removeRangeIntersectingAfterFirst tests passed!\r\n");
}
